"use strict";


/**
 * Hbbtv interface
 * @class Hbbtv
 * @namespace PluginInterface
 * @static
 */
PluginInterface.Hbbtv = new function() {
    /*
     ***********************************************************************************************************
     PRIVATE VARIABLES AND FUNCTIONS
     ***********************************************************************************************************
     */

    /** HBBTV callback
     * @method _hbbtv_callback
     * @private
     */
    var _hbbtv_callback = null,
    /** HBBTV ON/OFF state
     * @property _hbbtv_state
     * @private
     * @default 1 = ON
     */
    _hbbtv_state = 1,
    /** HBBTV visible state
     * @property _hbbtv_visible
     * @private
     * @default 1 = Visible
     */
    _hbbtv_visible = 1,
    /**
     * HBBTV callback handler
     * @private
     * @param keySet {Number} Value of Webkit key set
     * @method _fCallbackHbbtv
     */
    _fCallbackHbbtv = function (keySet) {
        setTimeout(function () {
            _fCallbackHbbtvHandling(keySet);
        }, 25);
    },
    /**
     * HBBTV media state callback
     * @private
     * @param state {Number} Value 0 AV decoder is used by HbbTV, 1 not used
     * @method _fCallbackHbbtvMediaState
     */
    _fCallbackHbbtvMediaState = function (state) {
        this.hbbtvAVDecoderStatus = state;
    },
    /**
     * HBBTV callback handling
     * @param keySet {Number} Value of Webkit key set
     * @method _fCallbackHbbtvHandling
     * @private
     */
    _fCallbackHbbtvHandling = function (keySet) {
        if (keySet != 0) {
            if (_hbbtv_callback) {
                _hbbtv_callback();
            }
        }
    };

    /*
     ***********************************************************************************************************
     PRIVILEGED PROPERTIES AND METHODS
     ***********************************************************************************************************
     */

    /**
     * Enumerated HBBTV On/OFF states
     * @property eHbbtvOnOff
     * @type {{OFF: number, ON: number}}
     * @final
     */
    this.eHbbtvOnOff = {
        OFF: 0,
        ON: 1
    };
    /**
     * Enumerated HBBTV visible states
     * @property eHbbtvVisible
     * @type {{OFF: number, ON: number}}
     * @final
     */
    this.eHbbtvVisible = {
        OFF: 0,
        ON: 1
    };

    /**
     * Initialize hbbtv module
     * @method fInitHbbtv
     * @static
     */
    this.fInitHbbtv = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.hbbtvNotification = _fCallbackHbbtv.bind(this);
        PluginInterface.Setup.stb.hbbPlaybackNotification = _fCallbackHbbtvMediaState.bind(this);
    };

    /**
     * Set Callbacks
     * @method fSetCallbacks
     * @param callback {Function} key handling callback
     * @static
     */
    this.fSetCallbacks = function (callback) {
        _hbbtv_callback = callback;
    };

    /**
     * Hide HBBTV
     * @method fHideHbbtv
     * @static
     */
    this.fHideHbbtv = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        if (_hbbtv_state == this.eHbbtvOnOff.ON && _hbbtv_visible == this.eHbbtvVisible.ON) {
            _hbbtv_visible = this.eHbbtvVisible.OFF;
            PluginInterface.Setup.stb.hideHbbTv("");
        }
    };

    /**
     * Show HBBTV
     * @method fShowHbbtv
     * @static
     */
    this.fShowHbbtv = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        if (_hbbtv_state == this.eHbbtvOnOff.ON && _hbbtv_visible == this.eHbbtvVisible.OFF) {
            _hbbtv_visible = this.eHbbtvVisible.ON;
            PluginInterface.Setup.stb.showHbbTv("");
        }
    };

    /**
     * Set HBBTV On
     * @method fSetHbbtvOn
     * @static
     */
    this.fSetHbbtvOn = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        if (_hbbtv_state == this.eHbbtvOnOff.OFF) {
            _hbbtv_state = this.eHbbtvOnOff.ON;
            _hbbtv_visible = this.eHbbtvVisible.ON;
            PluginInterface.Setup.stb.setHbbTvON("");
        }
    };

    /**
     * Set HBBTV Off
     * @method fSetHbbtvOff
     * @static
     */
    this.fSetHbbtvOff = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        if (_hbbtv_state == this.eHbbtvOnOff.ON) {
            _hbbtv_state = this.eHbbtvOnOff.OFF;
            _hbbtv_visible = this.eHbbtvVisible.OFF;
            PluginInterface.Setup.stb.setHbbTvOFF("");
        }
    };

    /**
     * Set HBBTV On or Off
     * @method fSetHbbtvOnOff
     * @param value {Number} Hbbtv off/on (0/1)
     * @static
     */
    this.fSetHbbtvOnOff = function (value, isPermanent) {
        isPermanent = typeof isPermanent == 'undefined' ? true : isPermanent;
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _hbbtv_state = value;
            return;
        }

        switch (parseInt(value, 10)) {
            case this.eHbbtvOnOff.OFF:
                _hbbtv_state = this.eHbbtvOnOff.OFF;
                _hbbtv_visible = this.eHbbtvVisible.OFF;
                PluginInterface.Setup.stb.setHbbTvOFF("");
                break;
            case this.eHbbtvOnOff.ON:
                _hbbtv_state = this.eHbbtvOnOff.ON;
                _hbbtv_visible = this.eHbbtvVisible.ON;
                // latest update: 30.05.2016
                PluginInterface.Setup.stb.setHbbTvON("");
                break;
        }
        if (isPermanent) {
            PluginInterface.Setup.stb.saveData(PluginInterface.Setup.DATA_HBBTV_ON_OFF, "" + value);
        }
    };

    /**
     * Get HBBTV On or Off
     * @method fGetHbbtvOnOff
     * @returns {Number} Hbbtv off/on (0/1)
     * @static
     */
    this.fGetHbbtvOnOff = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return _hbbtv_state;
        }
        var value = parseInt(PluginInterface.Setup.stb.loadData(PluginInterface.Setup.DATA_HBBTV_ON_OFF), 10);
        _hbbtv_state = (typeof value == 'undefined' || isNaN(value)) ? this.eHbbtvOnOff.ON : value;
        return _hbbtv_state;
    };

    /**
     * Set HBBTV Visible or Hidded
     * @method fSetHbbtvVisible
     * @param value {Number} Hbbtv 1 = visible, 0 = hidden
     * @static
     */
    this.fSetHbbtvVisible = function (value) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            _hbbtv_visible = value;
            return;
        }
        switch (parseInt(value, 10)) {
            case this.eHbbtvOnOff.OFF:
                _hbbtv_visible = this.eHbbtvVisible.OFF;
                PluginInterface.Setup.stb.hideHbbTv("");
                break;
            case this.eHbbtvOnOff.ON:
                _hbbtv_visible = this.eHbbtvVisible.ON;
                PluginInterface.Setup.stb.showHbbTv("");
                break;
        }
    };

    /**
     * Restart HBBTV
     * @method fRestartHbbtv
     * @static
     */
    this.fRestartHbbtv = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        // if global user settings -> hbbtv = ON only then restart hbbtv
        // if hbbtv  = off do not call restart as it would enable hbbtv
        if (_hbbtv_state == this.eHbbtvOnOff.ON) {
            PluginInterface.Setup.stb.restartHbbTv("");
        }
    };

    /**
     * UnFreeze HBBTV
     * @method fUNFreezeHbbtv
     * @static
     */
    this.fUNFreezeHbbtv = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.unFreezeHbbTV("");
    };

    /**
     * Freeze HBBTV
     * @method fFreezeHbbtv
     * @static
     */
    this.fFreezeHbbtv = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.freezeHbbTV("");
    };
};