"use strict";

ui.App.Callbacks = {
    entitlementMessage: {
        id: 0,
        frequency: 0,
        utc_time: 0
    },
    /**
     * Display network status message
     */
    networkStatusCallback: function (status) {
        switch (parseInt(status, 10)) {
            case PluginInterface.Network.networkStatus.NETWORK_UNPLUGGED:
                UIMiniFramework.Globals.isNetworkConnected = false;
                UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.networkUnplugged[UIMiniFramework.Globals.language]);
                break;
            case PluginInterface.Network.networkStatus.NETWORK_PLUGGED:
                UIMiniFramework.Globals.isNetworkConnected = true;
                UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.networkPlugged[UIMiniFramework.Globals.language]);
                break;
        }
        UIMiniFramework.Modules.Settings.screens.ethernetSettings.handleNetwork.call(UIMiniFramework.Modules.Settings.screens.ethernetSettings, parseInt(status, 10));
    },

    /**
     * Device notifications
     */
    deviceNotification: function (status, path, isPvr, isMounted, objPath, isPvrFormatted, devSize, devFullness, filesystem) {
        console.log('DEVICE NOTIFICATION: ' + status + " | " + path + " | " + isPvr + " | " + isMounted + " | " + objPath + " | " + isPvrFormatted + " | " + devSize + " | " + devFullness + " | " + filesystem);
        UIMiniFramework.Modules.UsbDevices.update(status, path, isPvr, isMounted, objPath, isPvrFormatted, devSize, devFullness, filesystem);
    },

    deviceFormatNotification: function (obj_path, format_state) {
        //console.log('FORMAT CALLBACK: ' + obj_path+" | "+format_state+" | "+ filesystem);
        UIMiniFramework.Modules.Confirmations.removeByCode('formatCode');
        if (parseInt(format_state, 10) === 0) {
            // error
            UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.mediaFormatFailed[UIMiniFramework.Globals.language]);
        }
        else {
            PluginInterface.MediaBrowsing.fMountUSB(obj_path, 1);
        }
        // set to 0 bacause notification will follow
        UIMiniFramework.Modules.UsbDevices.recorderCount = 0;
    },

    // only when unmount
    deviceMountNotification: function (objPath, mountState, isPvr, path, devSize, devFullness, filesystem) {
        //console.log('MOUNT CALLBACK : ' + objPath+" | "+mountState+" | " + isPvr + ' | ' + path+" | "+ devSize+" | "+ devFullness+" | "+ filesystem + ' | ' + name);
        var isPvrFormatted = PluginInterface.MediaBrowsing.deviceFormatState.NOT_PVR_FORMATTED;
        var name = UIMiniFramework.Modules.UsbDevices.getName(path);
        if (!name) {
            return;
        }

        switch (parseInt(mountState, 10)) {
            case PluginInterface.MediaBrowsing.deviceMountStatus.UNMOUNTED:
                if (parseInt(isPvr, 10) === PluginInterface.MediaBrowsing.deviceType.RECORDER) {
                    if ((UIMiniFramework.Config.MODULES.Pvr && !UIMiniFramework.Modules.Pvr.isCurrentlyRecording || !UIMiniFramework.Config.MODULES.Pvr) && (UIMiniFramework.Config.MODULES.Timeshift && !UIMiniFramework.Modules.Timeshift.isTimeshiftRecording || !UIMiniFramework.Config.MODULES.Timeshift)) {
                        UIMiniFramework.Modules.Notifications.add(name + ' ' + UIMiniFramework.Translations.OSD.systemMessages.OSD_021[UIMiniFramework.Globals.language]);
                    }
                    UIMiniFramework.Modules.UsbDevices.recorder.isMounted = false;
                }
                else {
                    UIMiniFramework.Modules.Notifications.add(name + ' ' + UIMiniFramework.Translations.OSD.systemMessages.OSD_021[UIMiniFramework.Globals.language]);
                    UIMiniFramework.Modules.UsbDevices.multimedia.isMounted = false;
                }
                UIMiniFramework.Modules.Settings.screens.mediaInformation.handleUsbDevice(PluginInterface.MediaBrowsing.deviceStatus.DISCONECTED, path, isPvr, mountState, objPath, isPvrFormatted, devSize, devFullness, filesystem, name);
                break;
            case PluginInterface.MediaBrowsing.deviceMountStatus.MOUNTED:
                if (parseInt(isPvr, 10) === PluginInterface.MediaBrowsing.deviceType.RECORDER) {
                    UIMiniFramework.Modules.UsbDevices.recorder.isMounted = true;
                }
                else {
                    UIMiniFramework.Modules.UsbDevices.multimedia.isMounted = true;
                }
                break;
        }

    },

    deviceBenchmarkNotification: function (obj_path, read_speed, write_speed) {
        //console.log(status+" | "+ path+" | "+ isPvr+" | "+ isMounted+" | "+ objPath+" | "+ isPvrFormatted+" | "+ devSize+" | "+ filesystem);
    },

    /**
     * Display service signal status
     */
    signalStatusCallback: function (signal) {
        console.log('SIGNAL STATUS: ' + signal);
        switch (parseInt(signal, 10)) {
            case PluginInterface.SignalStatus.signalStatus.SIGNAL_LOST:
                UIMiniFramework.Globals.isSignalAvailable = false;
                this.signalLostHandler();
                break;
            case PluginInterface.SignalStatus.signalStatus.SIGNAL_REGAINED:
                UIMiniFramework.Globals.isSignalAvailable = true;
                UIMiniFramework.Modules.PermanentInformations.removeByCode('standbyNspd');
                UIMiniFramework.Modules.PermanentInformations.removeByCode('signalStatusCode');
                break;
        }
    },

    signalLostHandler: function () {
        //check if teletext is running
        if (UIMiniFramework.Config.MODULES.Teletext && UIMiniFramework.Modules.Teletext.isCreated) {
            UIMiniFramework.Modules.Teletext.destroy();
        }
        UIMiniFramework.Modules.PermanentInformations.removeByCode('signalStatusCode');
        UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_102[UIMiniFramework.Globals.language], null, false, false, 'signalStatusCode');
        clearTimeout(UIMiniFramework.Modules.Hbbtv.hbbtvTimeoutId);
    },

    /**
     * handle zapper error
     */
    zapperErrorCallback: function (type) {
        console.log('ZAPPER ERROR CALLBACK | ' + type);
        UIMiniFramework.Globals.service.isZapped = true;
        switch (parseInt(type, 10)) {
            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_CHANNEL_CHANGE_FAIL:
                this.zapperCallback([], true);
                this.signalLostHandler();
                break;
            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_RUNNING_STATUS_CHANGE_RUNNING:
                UIMiniFramework.Modules.Confirmations.removeByCode('serviceNotRunningCode');
                break;
            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_RUNNING_STATUS_CHANGE_NOT_RUNNING:
                UIMiniFramework.Modules.Confirmations.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_015[UIMiniFramework.Globals.language], [], null, null, false, false, false, false, false, false, null, null, null, 'serviceNotRunningCode');
                break;
            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_DATA_MISSING_ON_PID:
                UIMiniFramework.Modules.Confirmations.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_016[UIMiniFramework.Globals.language], [], null, null, false, false, false, false, false, false, null, null, null, 'serviceNotRunningCode');
                break;
            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_SERVICE_CONNECTED:
                UIMiniFramework.Modules.PermanentInformations.removeByCode('serviceConnectCode');
                break;
            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_SERVICE_NOT_CONNECTED:
                UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_105[UIMiniFramework.Globals.language], null, false, false, 'serviceConnectCode');
                break;
            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_OCCASIONAL_UHD_START:
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_160[UIMiniFramework.Globals.language],
                    ['wechseln', 'verbleiben'],
                    function () { //yes
                        PluginInterface.ServiceZapper.fSetUhdEvent(1, 1);
                    },
                    null,
                    true,
                    false,
                    true,
                    null,
                    20000,
                    false
                );
                break;

            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_OCCASIONAL_UHD_END:
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_161[UIMiniFramework.Globals.language],
                    ['ja'],
                    function () { //yes
                        PluginInterface.ServiceZapper.fSetUhdEvent(1, 0);
                    },
                    function () { //no
                        PluginInterface.ServiceZapper.fSetUhdEvent(1, 0);
                    },
                    true,
                    true,
                    false,
                    20000,
                    null,
                    false
                );
                break;

            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_NO_INSTALLED_SERVICES:
                this.zapperCallback([], true);
                UIMiniFramework.Modules.Notifications.add('No installed services');
                break;
            case PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_SERVICE_NOT_EXIST:
                if (UIMiniFramework.Config.MODULES.InfoBanner && UIMiniFramework.Modules.InfoBanner.isCreated) {
                    UIMiniFramework.Modules.InfoBanner.updateChannelTitle(UIMiniFramework.Translations.error[UIMiniFramework.Globals.language]);
                    UIMiniFramework.Modules.InfoBanner.resetCloseTimeout();
                }
                break;
            default:
                break;
        }
    },

    cakCallback: function (event, data) {
        switch (parseInt(event, 10)) {
            case PluginInterface.Cak.casEvent.CAE_ERROR_POPUP:
                this.smartCardCallback(data.source, data.severity, data.status);
                break;
            case PluginInterface.Cak.casEvent.CAE_DISPLAY_POPUP:
                UIMiniFramework.Modules.PermanentInformations.removeByCode('cakPopup');
                UIMiniFramework.Modules.PermanentInformations.add(data.text, null, true, false, 'cakPopup');
                break;
            case PluginInterface.Cak.casEvent.CAE_REMOVE_POPUP:
                UIMiniFramework.Modules.PermanentInformations.removeByCode('cakPopup');
                break;

            case PluginInterface.Cak.casEvent.CAE_ENTITLEMENT_MESSAGE:
                this.entitlementMessageHandler(data);
                break;

            case PluginInterface.Cak.casEvent.CAE_CAS_ID_MISMATCH:
                UIMiniFramework.Modules.PermanentInformations.removeByCode('smartCardExtracted');
                UIMiniFramework.Modules.PermanentInformations.removeByCode('smartCardCode');
                UIMiniFramework.Modules.PermanentInformations.removeByCode('entitlementMessage');
                UIMiniFramework.Modules.PermanentInformations.removeByCode('closableEntitlementMessage');
                UIMiniFramework.Modules.PermanentInformations.removeByCode('scNotActivated');
                //system message 44
                UIMiniFramework.Modules.PermanentInformations.removeByCode('scIdMismatch');
                UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_044[UIMiniFramework.Globals.language], null, true, false, 'scIdMismatch');
                break;
            case PluginInterface.Cak.casEvent.CAE_BOOT_ERROR:
                if (UIMiniFramework.Modules.Input.currentFocus != null && UIMiniFramework.Modules.Input.currentFocus.screenClass == UIMiniFramework.Globals.modules.settings.ftiWelcome) {
                    UIMiniFramework.Modules.Settings.destroy()
                }
                ui.App.Init.standbyCallback(1);
                UIMiniFramework.Modules.FrontPanel.display('FUSES NOT SET');
                UIMiniFramework.Modules.PermanentInformations.add("Fuses not set.");
                break;
            case PluginInterface.Cak.casEvent.CAE_PERSISTENT_DATA:
                break;

            case PluginInterface.Cak.casEvent.CAE_SMARTCARD_NOT_ACTIVATED:
                //system message 26
                UIMiniFramework.Modules.PermanentInformations.removeByCode('scNotActivated');
                UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_026[UIMiniFramework.Globals.language], null, true, false, 'scNotActivated');
                break;
            case PluginInterface.Cak.casEvent.CAE_STATUS_CONTENT:
                if (data.smartcardState == PluginInterface.Cak.smartCardState.CAS_STATUS_SMARTCARD_EXTRACTED) {
                    var serviceData = PluginInterface.ServiceZapper.fGetServiceData(UIMiniFramework.Globals.service.activeServiceList, UIMiniFramework.Globals.service.activeChannelIndex);
                    /* In case of scrambled service, inform user that smart card is extracted. */
                    if (serviceData.scrambled == 1) {
                        var status = PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_EXTRACTED;
                        this.caAlarmProductionHandler(status);
                    }
                }
                break;

        }

    },

    smartCardCallback: function (source, severity, status) {
        //console.log('SMART CARD: ' + Number(source).toString(16) + ' | ' + severity + ' | ' + status);
        if (parseInt(source, 10) == 1) {
            switch (parseInt(status, 10)) {
                case PluginInterface.Cak.irqTypes.IRD_COMMAND_RESET_PIN:
                    UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_030[UIMiniFramework.Globals.language]);
                    PluginInterface.ParentalControl.fSetPinCode("0000");
                    break;
                case PluginInterface.Cak.irqTypes.IRD_COMMAND_RESTORE_FACTORY_SETTINGS:
                    UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_031[UIMiniFramework.Globals.language]);
                    setTimeout(PluginInterface.Setup.fResetFactoryDefaults.bind(PluginInterface.Setup), 5000);
                    break;
            }
        }
        else {
            this.caAlarmProductionHandler(status);
            //this.caAlarmCsoHandler(status);

        }
    },

    pinEntryCallback: function (lockType) {
        console.log('PIN ENTRY CALLBACK | ' + lockType);
        var message = '';
        switch (parseInt(lockType, 10)) {
            case PluginInterface.ParentalControl.eLockType.CHANNEL:
                message = UIMiniFramework.Translations.enterPin[UIMiniFramework.Globals.language];
                break;
            case PluginInterface.ParentalControl.eLockType.AGE:
                message = UIMiniFramework.Modules.PinEntry.getAgeLockMessage();
                break;
            case PluginInterface.ParentalControl.eLockType.PVR_AGE:
                message = UIMiniFramework.Modules.PinEntry.getAgeLockMessage();
                break;
        }
        UIMiniFramework.Modules.PinEntry.display.call(UIMiniFramework.Modules.PinEntry, UIMiniFramework.Modules.PinEntry.pinSuccessCallback, lockType, message);
    },

    pinExitCallback: function (lockType) {
        console.log('PIN EXIT = ' + lockType)
        switch (parseInt(lockType, 10)) {
            case PluginInterface.ParentalControl.eLockType.CHANNEL:
                UIMiniFramework.Modules.PinEntry.destroy.call(UIMiniFramework.Modules.PinEntry, lockType);
                UIMiniFramework.Globals.service.isServiceLocked = false;
                break;
            case PluginInterface.ParentalControl.eLockType.AGE:
                UIMiniFramework.Modules.PinEntry.destroy.call(UIMiniFramework.Modules.PinEntry, lockType);
                UIMiniFramework.Globals.service.isServiceAgeLocked = false;
                break;
            case PluginInterface.ParentalControl.eLockType.PVR_AGE:
                UIMiniFramework.Modules.PinEntry.destroy.call(UIMiniFramework.Modules.PinEntry, lockType);
                break;
        }
    },

    /**
     * This callback happens when CAK response SC pin validation
     * @param type
     * @param result
     */
    cakScPinCallback: function (type, result) {
        console.log('CAK SC PIN CALLBACK | ' + type + ' | ' + result);
        switch (parseInt(type, 10)) {
            case PluginInterface.Cak.pinType.PCT_MASTER:
            case PluginInterface.Cak.pinType.PCT_NEW:
            case PluginInterface.Cak.pinType.PCT_PAY_PER_VIEW:
                UIMiniFramework.Modules.Settings.screens.smartcardPinChange.pinCallback(type, result);
                break;
            case PluginInterface.Cak.pinType.PCT_PARENTAL_CONTROL:
                switch (parseInt(result, 10)) {
                    case PluginInterface.Cak.pinResultType.PCR_FAILED:
                    case PluginInterface.Cak.pinResultType.PCR_TYPE_INVALID:
                    case PluginInterface.Cak.pinResultType.PCR_BLOCK:
                        switch (UIMiniFramework.Globals.smartCardState) {
                            case 1:
                                UIMiniFramework.Globals.smartCardState = 2;
                                this.pinEntryCallback(UIMiniFramework.Modules.PinEntry.lockType);
                                break;
                            case 2:
                                UIMiniFramework.Globals.smartCardState = 3;
                                this.pinEntryCallback(UIMiniFramework.Modules.PinEntry.lockType);
                                break;
                            case 3:
                                UIMiniFramework.Globals.smartCardState = 4;
                                UIMiniFramework.Modules.PinEntry.pinBlocked.init();
                                this.pinEntryCallback(UIMiniFramework.Modules.PinEntry.lockType);
                                break;
                        }

                        break;

                    case PluginInterface.Cak.pinResultType.PCR_CHECK_OK:
                        UIMiniFramework.Globals.smartCardState = 1;
                        if (UIMiniFramework.Modules.PinEntry.lockType == PluginInterface.ParentalControl.eLockType.PVR_AGE) {
                            if (UIMiniFramework.Modules.PinEntry.pinSuccessCallback) {
                                UIMiniFramework.Modules.PinEntry.pinSuccessCallback();
                            }
                        }

                        break;
                    case PluginInterface.Cak.pinResultType.PCR_CHANGED:
                    case PluginInterface.Cak.pinResultType.PCR_FAIL_CARD_OUT:
                        break;
                }
                break;
        }
    },

    caAlarmProductionHandler: function (status) {
        console.log('caAlarmProductionHandler | STATUS =  ' + status);
        //todo: get real values
        var hotline_number = "0900-1904599";
        var call_charge = "0.49";
        var smartcard_no = null;
        var pairing_no = null;
        var info = null;

        //clear previous notifications
        UIMiniFramework.Modules.PermanentInformations.removeByCode('smartCardCode');
        UIMiniFramework.Modules.PermanentInformations.removeByCode('smartCardExtracted');

        switch (parseInt(status, 10)) {
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_CLEAR:
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_GRANTED:
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_FREE:
                //already removed on upper line

                //remove system message 26 on card activation
                UIMiniFramework.Modules.PermanentInformations.removeByCode('scNotActivated');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_ERROR:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.caAlarm.OSD_9[UIMiniFramework.Globals.language],
                    {
                        $hotline_number$: hotline_number,
                        $call_charge$: call_charge
                    },
                    false,
                    false,
                    'smartCardCode'
                );
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_MUTE:
                UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.caAlarm.OSD_10a[UIMiniFramework.Globals.language], null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_INVALID:
                UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.caAlarm.OSD_11[UIMiniFramework.Globals.language], null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_BLACKLISTED:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.caAlarm.OSD_12[UIMiniFramework.Globals.language],
                    {
                        $hotline_number$: hotline_number,
                        $call_charge$: call_charge
                    },
                    false,
                    false,
                    'smartCardCode'
                );
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_SUSPENDED:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.caAlarm.OSD_13[UIMiniFramework.Globals.language],
                    {
                        $hotline_number$: hotline_number
                    },
                    false,
                    false,
                    'smartCardCode'
                );
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_EXPIRED:
                UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.caAlarm.OSD_14[UIMiniFramework.Globals.language], null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED:
                info = PluginInterface.Cak.fGetCakInfo();
                smartcard_no = info.smartcardSerial;
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.caAlarm.OSD_15[UIMiniFramework.Globals.language],
                    {
                        $smartcard_no$: smartcard_no
                    },
                    false,
                    false,
                    'smartCardCode'
                );
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_NOT_CERTIFIED:
                UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.caAlarm.OSD_16[UIMiniFramework.Globals.language], null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_NEVER_PAIRED:
                UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.caAlarm.OSD_17[UIMiniFramework.Globals.language], null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_NOT_PAIRED:
                info = PluginInterface.Cak.fGetCakInfo();
                pairing_no = info.irdSerialNumber;
                smartcard_no = info.smartcardSerial;
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.caAlarm.OSD_18[UIMiniFramework.Globals.language],
                    {
                        $hotline_number$: hotline_number,
                        $call_charge$: call_charge,
                        $smartcard_no$: smartcard_no,
                        $pairing_no$: pairing_no
                    },
                    false,
                    false,
                    'smartCardCode'
                );
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_CHIPSET_PAIRING_REQUIRED:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.caAlarm.OSD_18[UIMiniFramework.Globals.language],
                    {
                        $hotline_number$: hotline_number,
                        $call_charge$: call_charge
                    },
                    false,
                    false,
                    'smartCardCode'
                );
                break;
            //defined as system message (not defined as CA alarm ??) in AS-3101
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_EXTRACTED:
                //remove all entitlement messages if sc is extracted
                UIMiniFramework.Modules.PermanentInformations.removeByCode('entitlementMessage');
                UIMiniFramework.Modules.PermanentInformations.removeByCode('closableEntitlementMessage');
                UIMiniFramework.Modules.PermanentInformations.removeByCode('scNotActivated');
                UIMiniFramework.Modules.PermanentInformations.removeByCode('scIdMismatch');
                UIMiniFramework.Modules.PermanentInformations.add(UIMiniFramework.Translations.OSD.caAlarm.OSD_20[UIMiniFramework.Globals.language], null, false, false, 'smartCardExtracted');
                break;
            //not defined in AS-3101 System_Messages_CAK_V7_v-1-6
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_INCOMPATIBLE:
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_BLACKED_OUT:
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_NO_VALID_CREDIT:
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_COPY_PROTECTED:
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_PARENTAL_CONTROL:
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_DIALOG_REQUIRED:
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_PAIRING_REQUIRED:
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_UNKNOW_ERROR:
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SECURITY_UPGRADE_REQUIRED:
                break;
        }
    },

    caAlarmCsoHandler: function (status) {
        //clear previous notifications
        UIMiniFramework.Modules.PermanentInformations.removeByCode('smartCardCode');
        UIMiniFramework.Modules.PermanentInformations.removeByCode('smartCardExtracted');

        switch (parseInt(status, 10)) {
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_CLEAR:
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_GRANTED:
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_FREE:
                //already removed on upper line
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_ERROR:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_ERROR", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_MUTE:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_MUTE", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_INVALID:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_INVALID", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_BLACKLISTED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_BLACKLISTED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_SUSPENDED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_SUSPENDED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_EXPIRED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_EXPIRED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_ACCESS_DENIED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_NOT_CERTIFIED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_NOT_CERTIFIED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_NEVER_PAIRED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_NEVER_PAIRED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_NOT_PAIRED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_NOT_PAIRED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_CHIPSET_PAIRING_REQUIRED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_ACCESS_DENIED_CHIPSET_PAIRING_REQUIRED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_INCOMPATIBLE:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_INCOMPATIBLE", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_BLACKED_OUT:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_ACCESS_BLACKED_OUT", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_NO_VALID_CREDIT:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_ACCESS_DENIED_NO_VALID_CREDIT", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_COPY_PROTECTED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_ACCESS_DENIED_COPY_PROTECTED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_PARENTAL_CONTROL:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_ACCESS_DENIED_PARENTAL_CONTROL", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_DIALOG_REQUIRED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_ACCESS_DENIED_DIALOG_REQUIRED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_ACCESS_DENIED_PAIRING_REQUIRED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_ACCESS_DENIED_PAIRING_REQUIRED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_UNKNOW_ERROR:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_UNKNOW_ERROR", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SECURITY_UPGRADE_REQUIRED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SECURITY_UPGRADE_REQUIRED", null, false, false, 'smartCardCode');
                break;
            case PluginInterface.Cak.errorStatusTypes.STATUS_SMARTCARD_EXTRACTED:
                UIMiniFramework.Modules.PermanentInformations.add("STATUS_SMARTCARD_EXTRACTED", null, false, false, 'smartCardCode');
                break;
        }
    },

    entitlementMessageHandler: function (data) {
        UIMiniFramework.Modules.PermanentInformations.removeByCode('entitlementMessage');
        UIMiniFramework.Modules.PermanentInformations.removeByCode('closableEntitlementMessage');

        this.entitlementMessage.id = parseInt(data.id, 10);
        this.entitlementMessage.frequency = parseInt(data.display_frequency, 10);
        this.entitlementMessage.utc_time = parseInt(data.utc_time, 10);

        // 30, 32, 34, 35 and 36 shall disappear upon manual confirmation or exit button
        switch (this.entitlementMessage.id) {
            case 30:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.entitlementMessage.OSD_30[UIMiniFramework.Globals.language],
                    {
                        $expire_days$: data.expire_in_x_days,
                        $smartcard_no$: data.smartcard_sn,
                        $hotline_number$: data.hotline_number,
                        $call_charge$: data.call_charge
                    },
                    true,
                    false,
                    'closableEntitlementMessage'
                );
                break;
            case 31:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.entitlementMessage.OSD_31[UIMiniFramework.Globals.language],
                    {
                        $smartcard_no$: data.smartcard_sn,
                        $hotline_number$: data.hotline_number,
                        $call_charge$: data.call_charge
                    },
                    true,
                    false,
                    'entitlementMessage'
                );
                break;
            case 32:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.entitlementMessage.OSD_32[UIMiniFramework.Globals.language],
                    {
                        $expire_days$: data.expire_in_x_days,
                        $smartcard_no$: data.smartcard_sn,
                        $hotline_number$: data.hotline_number,
                        $call_charge$: data.call_charge
                    },
                    true,
                    false,
                    'closableEntitlementMessage'
                );
                break;
            case 33:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.entitlementMessage.OSD_33[UIMiniFramework.Globals.language],
                    {
                        $hotline_number$: data.hotline_number,
                        $call_charge$: data.call_charge
                    },
                    true,
                    false,
                    'entitlementMessage'
                );
                break;
            case 34:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.entitlementMessage.OSD_34[UIMiniFramework.Globals.language],
                    {
                        $smartcard_no$: data.smartcard_sn,
                        $hotline_number$: data.hotline_number,
                        $call_charge$: data.call_charge
                    },
                    true,
                    false,
                    'closableEntitlementMessage'
                );
                break;
            case 35:
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.entitlementMessage.OSD_35[UIMiniFramework.Globals.language],
                    {
                        $hotline_number$: data.hotline_number,
                        $call_charge$: data.call_charge
                    },
                    true,
                    false,
                    'closableEntitlementMessage'
                );
                break;
            case 36:
                var d = new Date(data.expire_date * 1000);
                UIMiniFramework.Modules.PermanentInformations.add(
                    UIMiniFramework.Translations.OSD.entitlementMessage.OSD_36[UIMiniFramework.Globals.language],
                    {
                        $smartcard_no$: data.smartcard_sn,
                        $expire_date$: d.getDate() + "/" + (d.getMonth() + 1) + "/" + d.getFullYear()
                    },
                    true,
                    false,
                    'closableEntitlementMessage'
                );
                break;
        }
    },

    confirmEntitlementMessage: function () {
        if (this.entitlementMessage.id == 0) return;
        PluginInterface.Cak.fConfirmEntitlementMessage(this.entitlementMessage.id, this.entitlementMessage.frequency, this.entitlementMessage.utc_time);
        this.entitlementMessage.id = 0;
        this.entitlementMessage.frequency = 0;
        this.entitlementMessage.utc_time = 0;
    },

    reminderCallback: function (data) {
        var serviceLcn = parseInt(data.lcnServiceIndex, 10);
        var serviceList = parseInt(data.lcnListIndex, 10);
        var serviceIndex = UIMiniFramework.ServiceListCache.serviceListLcn[serviceList].indexOf(serviceLcn);
        var serviceName = UIMiniFramework.ServiceListCache.serviceList[serviceList][serviceIndex].name;
        UIMiniFramework.Modules.Confirmations.add(
            UIMiniFramework.Translations.OSD.systemMessages.OSD_350.part1[UIMiniFramework.Globals.language]
            + '"' + serviceName + '"'
            + UIMiniFramework.Translations.OSD.systemMessages.OSD_350.part2[UIMiniFramework.Globals.language],
            null,
            function () { //yes
                if (UIMiniFramework.Modules.MediaPlayer.isCreated) { //in media playback
                    PluginInterface.MediaPlayback.fStop();
                }
                UIMiniFramework.Modules.ZapperWrapper.setActiveService(serviceList, serviceIndex);
                PluginInterface.Epg.fScaleLiveRoute(0, 0, 0, 1280, 720);
            },
            null,
            true,
            true,
            false,
            60000,
            null,
            false
        );
    },

    /**
     * present following update callback
     */
    presentFollowingUpdateCallback: function () {
        console.log('PRESENT FOLLOWING UPDATE CALLBACK .............................');
        if (UIMiniFramework.Config.MODULES.InfoBanner && UIMiniFramework.Modules.InfoBanner.isCreated && !UIMiniFramework.Globals.service.presentFollowingDataAvailable) {
            // update p/f data from callback only if we'r positioned on active service (if shifted up/down do not because this data are related to active service only)
            if (UIMiniFramework.Globals.service.channelIndex == UIMiniFramework.Globals.service.activeChannelIndex) {
                UIMiniFramework.Modules.InfoBanner.resetCloseTimeout.call(UIMiniFramework.Modules.InfoBanner);
                UIMiniFramework.Modules.InfoBanner.updateEventData.call(UIMiniFramework.Modules.InfoBanner, PluginInterface.ServiceZapper.eventType.PRESENT, true);
            }
        }

        if (UIMiniFramework.Config.MODULES.Pvr && UIMiniFramework.Modules.Pvr.banner.isCreated) {
            UIMiniFramework.Modules.Pvr.banner.updateData();
        }
    },

    createTimerRecordResponse: function (value) {
        switch (parseInt(value, 10)) {
            case PluginInterface.Pvr.eScheduleRecordingStatus.SUCCESS:
                UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_300[UIMiniFramework.Globals.language]);
                if (UIMiniFramework.Modules.Scheduler.isCreated) {
                    UIMiniFramework.Modules.Scheduler.refresh();
                }
                break;
            case PluginInterface.Pvr.eScheduleRecordingStatus.ERROR:
                UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_301[UIMiniFramework.Globals.language]);
                break;
            case PluginInterface.Pvr.eScheduleRecordingStatus.REMOVED:
                UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_306[UIMiniFramework.Globals.language]);
                break;
        }
    },

    setReminderDataResponse: function (value) {
        switch (parseInt(value, 10)) {
            case PluginInterface.Reminder.eScheduleReminderStatus.SUCCESS:
                UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_302[UIMiniFramework.Globals.language]);
                break;
            case PluginInterface.Reminder.eScheduleReminderStatus.ERROR_GENERAL:
            case PluginInterface.Reminder.eScheduleReminderStatus.ERROR_NOT_INITIALIZED:
            case PluginInterface.Reminder.eScheduleReminderStatus.ERROR_ALREADY_INITIALIZED:
            case PluginInterface.Reminder.eScheduleReminderStatus.ERROR_BAD_ARGUMENT:
                UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_303[UIMiniFramework.Globals.language]);
                break;
            case PluginInterface.Reminder.eScheduleReminderStatus.ERROR_LIST_FULL:
                UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_304[UIMiniFramework.Globals.language]);
                break;
            case PluginInterface.Reminder.eScheduleReminderStatus.REMOVED:
                UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_305[UIMiniFramework.Globals.language]);
                break;
        }
    },

    update: {
        swUpdatePostponeCallback: null,

        callback: function (data) {
            UIMiniFramework.Modules.Loader.toggleLoader(false);
            this.statusHandler(data, true);
        },

        updateAvailableHandler: function (data) {
            UIMiniFramework.Modules.Confirmations.add(
                UIMiniFramework.Translations.OSD.systemMessages.OSD_150[UIMiniFramework.Globals.languages.items.GERMAN],
                [
                    UIMiniFramework.Translations.yes[UIMiniFramework.Globals.languages.items.GERMAN],
                    UIMiniFramework.Translations.no[UIMiniFramework.Globals.languages.items.GERMAN]
                ],
                function () {
                    ui.App.Callbacks.update.proceedToUpdate(data)
                },
                function () {
                    ui.App.Callbacks.update.reject(true)
                },
                false,
                false,
                false,
                null,
                null,
                false
            );
        },

        updateAvailableForcedHandler: function (data) {
            if (PluginInterface.Setup.fIsMaxPostponeUpdateReached()) {
                //yes->information (forced update available and already postponed maximum times)
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_152[UIMiniFramework.Globals.languages.items.GERMAN],
                    [
                        UIMiniFramework.Translations.startTheUpdate[UIMiniFramework.Globals.languages.items.GERMAN] + ' [' + UIMiniFramework.Translations.ok[UIMiniFramework.Globals.languages.items.GERMAN] + ']'
                    ],
                    function () {
                        ui.App.Callbacks.update.proceedToUpdate(data)
                    },  //accept update
                    null,
                    false,
                    false,
                    false,
                    null,
                    null,
                    false,
                    null,
                    null,
                    null,
                    'ignoreExit'
                );
            }
            else {//no->confirmation (forced update available)
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_151[UIMiniFramework.Globals.languages.items.GERMAN],
                    [
                        UIMiniFramework.Translations.now[UIMiniFramework.Globals.languages.items.GERMAN],
                        UIMiniFramework.Translations.later[UIMiniFramework.Globals.languages.items.GERMAN]
                    ],
                    function () {
                        ui.App.Callbacks.update.proceedToUpdate(data)
                    },  //accept update
                    function () {
                        ui.App.Callbacks.update.reject(false)
                    }, //reject update
                    false,
                    false,
                    false,
                    null,
                    null,
                    false
                );
            }
        },

        statusHandler: function (data, showNotificationForNoUpdate) {
            switch (parseInt(data.event, 10)) {
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_NO_UPDATE:
                    if (showNotificationForNoUpdate) {
                        var message = '';
                        switch (UIMiniFramework.Globals.softwareUpdateState) {
                            case UIMiniFramework.Globals.modules.settings.swUpgradeViaUsb:
                                message = UIMiniFramework.Translations.popups.noNewSoftwareAvailable[UIMiniFramework.Globals.language];
                                break;
                            case UIMiniFramework.Globals.modules.settings.swUpgradeViaSatellite:
                                message = UIMiniFramework.Translations.popups.newSoftwareNotDetected[UIMiniFramework.Globals.language];
                                break;
                        }
                        UIMiniFramework.Modules.ZapperWrapper.setLastPlayedService();
                        UIMiniFramework.Modules.Confirmations.add(
                            message,
                            [],
                            null,
                            null,
                            false,
                            false,
                            false,
                            null,
                            null,
                            false,
                            UIMiniFramework.Translations.information[UIMiniFramework.Globals.languages.items.GERMAN]
                        );
                    }
                    break;
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UPDATE_AVAILABLE:
                    if (UIMiniFramework.Globals.isServiceListUpdatedInStandby) {
                        ui.App.Callbacks.update.swUpdatePostponeCallback = ui.App.Callbacks.update.updateAvailableHandler.bind(ui.App.Callbacks.update, data);
                        return;
                    }
                    ui.App.Callbacks.update.swUpdatePostponeCallback = null;
                    ui.App.Callbacks.update.updateAvailableHandler(data);
                    break;
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UPDATE_AVAILABLE_FORCED:
                    if (UIMiniFramework.Globals.isServiceListUpdatedInStandby) {
                        ui.App.Callbacks.update.swUpdatePostponeCallback = ui.App.Callbacks.update.updateAvailableForcedHandler.bind(ui.App.Callbacks.update, data);
                        return;
                    }
                    ui.App.Callbacks.update.swUpdatePostponeCallback = null;
                    ui.App.Callbacks.update.updateAvailableForcedHandler(data);
                    break;
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UPDATE_ACCEPTED:
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UPDATE_INSTALLED:
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UPDATE_PENDING:
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UPDATE_FAILED:
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UP_TO_DATE:
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_PROGRESS_CHANGED:
                case PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UNDEFINED:
            }
        },

        proceedToUpdate: function (data) {
            switch (UIMiniFramework.Globals.softwareUpdateState) {
                case UIMiniFramework.Globals.modules.settings.swUpgradeViaUsb:
                    PluginInterface.Setup.fAcceptSsuUpdate(true);
                    break;
                case UIMiniFramework.Globals.modules.settings.swUpgradeViaSatellite:
                default:
                    //uncomment for OTA only update
                    //PluginInterface.Setup.fAcceptSsuUpdate(true);

                    //uncomment for IP/OTA updates handling
                    ui.App.Callbacks.update.ipOtaUpdateAcceptHandler(data);
                    break;
            }
        },
        reject: function (showMessage) {
            if (showMessage) {
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.softwareUpdateNoMoreNotify[UIMiniFramework.Globals.language],
                    [UIMiniFramework.Translations.ok[UIMiniFramework.Globals.language]],
                    null,
                    null,
                    false,
                    false,
                    false,
                    null,
                    null,
                    false,
                    UIMiniFramework.Translations.information[UIMiniFramework.Globals.language]
                );
            }
            UIMiniFramework.Modules.ZapperWrapper.setLastPlayedService();
            PluginInterface.Setup.fAcceptSsuUpdate(false);
        },

        ipOtaUpdateAcceptHandler: function (data) {
            if (parseInt(data.mode, 10) != PluginInterface.Setup.eSSUSoftwareUpdateMode.MAL_SSU_US_OTA) { //update over ip available
                this.callCheckConnection(data);
            }
            else {
                PluginInterface.Setup.fAcceptSsuUpdate(true, PluginInterface.Setup.eSSUSoftwareUpdateMode.MAL_SSU_US_OTA);
            }
        },
        callCheckConnection: function (data) {
            UIMiniFramework.Modules.Loader.toggleLoader(true);
            setTimeout(ui.App.Callbacks.update.checkConnection, 10, data);
        },
        checkConnection: function (data) {
            PluginInterface.Setup.fSsuSetEthConnectedStatus(UIMiniFramework.Globals.isNetworkConnected);
            if (UIMiniFramework.Globals.isNetworkConnected && PluginInterface.MediaBrowsing.fDetectHttp(data.url) == true) { //STB is connected
                UIMiniFramework.Modules.Loader.toggleLoader(false);
                PluginInterface.Setup.fAcceptSsuUpdate(true, PluginInterface.Setup.eSSUSoftwareUpdateMode.MAL_SSU_US_IP);
            }
            else { //STB isn't connected
                UIMiniFramework.Modules.Loader.toggleLoader(false);
                switch (data.mode) {
                    case PluginInterface.Setup.eSSUSoftwareUpdateMode.MAL_SSU_US_BOTH:
                        UIMiniFramework.Modules.Confirmations.add(
                            UIMiniFramework.Translations.connectBoxToInternet[UIMiniFramework.Globals.language] + '<br/>' + UIMiniFramework.Translations.updateOverSateliteTwoHours[UIMiniFramework.Globals.language],   //TODO: add german and turkish translations
                            [UIMiniFramework.Translations.retry[UIMiniFramework.Globals.language], UIMiniFramework.Translations.next[UIMiniFramework.Globals.language]],
                            function () {
                                ui.App.Callbacks.update.callCheckConnection(data)
                            },
                            function () {
                                ui.App.Callbacks.update.checkForcedUpdateAgain(data)
                            },
                            false,
                            false,
                            false,
                            null,
                            null,
                            false,
                            UIMiniFramework.Translations.information[UIMiniFramework.Globals.language]
                        );
                        break;
                    case PluginInterface.Setup.eSSUSoftwareUpdateMode.MAL_SSU_US_IP:
                        //check if forced
                        if (parseInt(data.event, 10) == PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UPDATE_AVAILABLE_FORCED) {
                            UIMiniFramework.Modules.Confirmations.add(
                                UIMiniFramework.Translations.connectBoxToInternet[UIMiniFramework.Globals.language],
                                [UIMiniFramework.Translations.retry[UIMiniFramework.Globals.language], UIMiniFramework.Translations.cancel[UIMiniFramework.Globals.language]],
                                function () {
                                    ui.App.Callbacks.update.callCheckConnection(data)
                                },
                                function () {
                                    PluginInterface.Setup.fAcceptSsuUpdate(false)
                                },
                                false,
                                false,
                                false,
                                null,
                                null,
                                false,
                                UIMiniFramework.Translations.information[UIMiniFramework.Globals.language]
                            );
                            break;
                        }
                        else {
                            ui.App.Callbacks.update.showUSBAvailableMsg(data.url);
                        }
                }
            }
        },

        checkForcedUpdateAgain: function (data) {
            if (parseInt(data.event, 10) == PluginInterface.Setup.eSsuEvents.MAL_SSU_EVENT_UPDATE_AVAILABLE_FORCED) {
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.updateOverSateliteTwoHours[UIMiniFramework.Globals.language],   //TODO: add german and turkish translations
                    [UIMiniFramework.Translations.ok[UIMiniFramework.Globals.language]],
                    function () {
                        PluginInterface.Setup.fAcceptSsuUpdate(true, PluginInterface.Setup.eSSUSoftwareUpdateMode.MAL_SSU_US_OTA);
                    },
                    null,
                    false,
                    false,
                    false,
                    null,
                    null,
                    false,
                    UIMiniFramework.Translations.information[UIMiniFramework.Globals.language],
                    null,
                    null,
                    'ignoreExit'
                );
            }
            else {
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.updateOverSateliteTwoHours[UIMiniFramework.Globals.language] + ' ' + UIMiniFramework.Translations.proceed[UIMiniFramework.Globals.language],  //TODO: add german and turkish translations
                    [UIMiniFramework.Translations.yes[UIMiniFramework.Globals.language], UIMiniFramework.Translations.no[UIMiniFramework.Globals.language]],
                    function () {
                        PluginInterface.Setup.fAcceptSsuUpdate(true, PluginInterface.Setup.eSSUSoftwareUpdateMode.MAL_SSU_US_OTA);
                    },
                    function () {
                        ui.App.Callbacks.update.showUSBAvailableMsg(data.url);
                    },
                    false,
                    false,
                    false,
                    null,
                    null,
                    false,
                    UIMiniFramework.Translations.information[UIMiniFramework.Globals.language]
                );
            }
        },
        showUSBAvailableMsg: function (url) {
            UIMiniFramework.Modules.Confirmations.add(
                UIMiniFramework.Translations.updateOverUsbAvailableLink[UIMiniFramework.Globals.language],
                [UIMiniFramework.Translations.ok[UIMiniFramework.Globals.language]],
                null,
                null,
                false,
                false,
                false,
                null,
                null,
                false,
                UIMiniFramework.Translations.information[UIMiniFramework.Globals.language]
            );
        }
    },

    serviceListUpdateCallback: function (status) {
        console.log('SERVICE LIST UPDATE CALLBACK | ' + status);
        //init service list cache
        UIMiniFramework.ServiceListCache.init();
        switch (parseInt(status, 10)) {
            case PluginInterface.ServiceZapper.updateServiceListStatus.AFTER_REBOOT: // after reboot
                var lastServiceList = PluginInterface.ServiceZapper.getLastActiveServiceList();
                var lastServiceIndex = PluginInterface.ServiceZapper.fGetLastPlayedService();
                var totalInstalledServices = PluginInterface.ServiceZapper.fGetNumberOfServices(PluginInterface.ServiceZapper.serviceList.MASTER_LIST);
                var totalServicesOnLastList = PluginInterface.ServiceZapper.fGetNumberOfServices(lastServiceList);
                if (parseInt(totalInstalledServices, 10) == 0 || lastServiceIndex >= totalServicesOnLastList) {
                    UIMiniFramework.Modules.Input.lockInput(false);
                    ui.App.Callbacks.zapperErrorCallback(PluginInterface.ServiceZapper.updateStatusEvent.DBUS_SZ_EVENT_NO_INSTALLED_SERVICES);
                    // do not set active service neither trigger callback_fast_info_zapping if no services installed
                    return false;
                }

                if (UIMiniFramework.Globals.serviceListReorder.isReordering) {
                    UIMiniFramework.Modules.Settings.screens.channelListEditor.afterReordering.call(UIMiniFramework.Modules.Settings.screens.channelListEditor);
                }
                else {
                    UIMiniFramework.Modules.ZapperWrapper.setLastPlayedService(true);
                    UIMiniFramework.Modules.Input.lockInput(false);
                }
                break;
            case PluginInterface.ServiceZapper.updateServiceListStatus.AFTER_SCAN: // after scan
                if (UIMiniFramework.Globals.service.setDefaultActiveService) {
                    console.log('SERVICE LIST UPDATE CALLBACK - SET ACTIVE SERVICE');
                    UIMiniFramework.Globals.service.setDefaultActiveService = false;
                    UIMiniFramework.Globals.service.isActiveServiceStopped = false;
                    setTimeout(function () {
                        UIMiniFramework.Modules.ZapperWrapper.setActiveService(PluginInterface.ServiceZapper.fGetDefaultServiceList(), 0);
                        UIMiniFramework.Modules.Input.lockInput(false);
                    }, 220);
                }
                break;
            case PluginInterface.ServiceZapper.updateServiceListStatus.AFTER_STANDBY: // after standby
                UIMiniFramework.Modules.Confirmations.removeByCode('wantToFormatCode');
                this.serviceListUpdatedInStandbyHandler();
                break;
            case PluginInterface.ServiceZapper.updateServiceListStatus.AFTER_REBOOT_AND_CHANNEL_LIST_UPDATE: // after reboot and channel list update
                UIMiniFramework.Modules.Confirmations.removeByCode('wantToFormatCode');
                this.serviceListUpdatedInStandbyHandler();
                setTimeout(function () {
                    PluginInterface.ServiceZapper.fPlayLastService();
                }, 220);
                break;
            case PluginInterface.ServiceZapper.updateServiceListStatus.REPLACED_SD_WITH_HD_SERVICES: //
                UIMiniFramework.Modules.Confirmations.removeByCode('replacedSDWithHDservice');
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_171[UIMiniFramework.Globals.language],
                    [],
                    null,
                    null,
                    false,
                    false,
                    false,
                    false,
                    false,
                    false,
                    UIMiniFramework.Translations.information[UIMiniFramework.Globals.language],
                    null,
                    'replacedSDWithHDservice',
                    'replacedSDWithHDservice'
                );
                break;
            case PluginInterface.ServiceZapper.updateServiceListStatus.REPLACED_SD_OR_HD_WITH_UHD_SERVICES: //
                UIMiniFramework.Modules.Confirmations.removeByCode('replacedSdOrHDWithUHDService');
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_172[UIMiniFramework.Globals.language],
                    [],
                    null,
                    null,
                    false,
                    false,
                    false,
                    false,
                    false,
                    false,
                    UIMiniFramework.Translations.information[UIMiniFramework.Globals.language],
                    null,
                    'replacedSdOrHDWithUHDService',
                    'replacedSdOrHDWithUHDService'
                );
                break;
        }
    },

    serviceListUpdatedInStandbyHandler: function () {
        UIMiniFramework.Modules.Loader.toggleLoader(false);
        UIMiniFramework.Modules.Confirmations.add(
            UIMiniFramework.Translations.OSD.systemMessages.OSD_170[UIMiniFramework.Globals.language],
            [],
            null,
            null,
            false,
            false,
            false,
            false,
            false,
            false,
            UIMiniFramework.Translations.information[UIMiniFramework.Globals.language],
            null,
            'serviceListUpdatedInStandby',
            'serviceListUpdatedInStandby'
        );
    },

    stateChangeCallback: function (data) {
        if (parseInt(data, 10) === 1) {
            UIMiniFramework.Modules.ZapperWrapper.setLastPlayedService();
        }
    },

    //* 0 = notify user to stop recording if he wants to change service,
    //* 1 = ask user if he wants to stop live in order to start recording
    routeManagerCallback: function (event) {
        console.log('route manager callback ' + event);
        switch (parseInt(event, 10)) {
            case PluginInterface.ServiceZapper.routeManagerEvents.DBUS_ROUTE_EVENT_NOTIFY_STOP_RECORDING:
                UIMiniFramework.Modules.Pvr.whileRecordingCantZapInformation.call(UIMiniFramework.Modules.Pvr);
                break;
            case PluginInterface.ServiceZapper.routeManagerEvents.DBUS_ROUTE_EVENT_STOP_LIVE_YES_OR_NO:
                break;
            default:
                break;
        }
    },

    recordingZappCallback: function (serviceIndex, listIndex, isDigitZapp) {
        isDigitZapp = typeof isDigitZapp == 'undefined' ? false : isDigitZapp;
        listIndex = typeof listIndex == 'undefined' || listIndex < 0 ? UIMiniFramework.Globals.service.activeServiceList : listIndex;
        console.log('recording zapp callback | index = ' + serviceIndex + ' | list index = ' + listIndex);
        UIMiniFramework.Modules.Pvr.recordingZappServiceIndex = parseInt(serviceIndex, 10);
        UIMiniFramework.Modules.Pvr.recordingZappListIndex = parseInt(listIndex, 10);
        UIMiniFramework.Globals.service.isZapped = true;
    },

    // Ask user if user wants to zap to serviceIndex in order to record the event that was scheduled
    pvrScheduledRecordingCallback: function (recordIndex, progHandle, isStarting) {
        console.log('pvr scheduled recording callback : ' + recordIndex + ' | ' + progHandle + ' | ' + isStarting);
        if (!UIMiniFramework.Config.MODULES.Pvr){
            return;
        }
        var currentRecordIndex = recordIndex; //PluginInterface.Pvr.fGetScheduledRecordIndex();
        var data = PluginInterface.Pvr.fGetScheduledRecord(currentRecordIndex);
        var eventName = data.name;
        var lcn = parseInt(data.lcnServiceIndex, 10);
        var listIndex = parseInt(data.lcnListIndex, 10);
        var serviceIndex = UIMiniFramework.ServiceListCache.serviceListLcn[listIndex].indexOf(lcn);
        var serviceName = UIMiniFramework.ServiceListCache.serviceList[listIndex][serviceIndex].name;

        if (parseInt(isStarting, 10) == 1) {
            UIMiniFramework.Modules.Confirmations.add(
                UIMiniFramework.Translations.OSD.systemMessages.OSD_355.part1[UIMiniFramework.Globals.language] + eventName +
                UIMiniFramework.Translations.OSD.systemMessages.OSD_355.part2[UIMiniFramework.Globals.language] + serviceName +
                UIMiniFramework.Translations.OSD.systemMessages.OSD_355.part3[UIMiniFramework.Globals.language],
                null,
                function () {
                    UIMiniFramework.Modules.Pvr.scheduledRecording.isTriggered = true;
                    UIMiniFramework.Modules.Pvr.scheduledRecording.serviceIndex = serviceIndex;
                    UIMiniFramework.Modules.Pvr.scheduledRecording.listIndex = listIndex;
                    UIMiniFramework.Modules.Pvr.scheduledRecording.progHandle = progHandle;
                    UIMiniFramework.Modules.ZapperWrapper.setActiveService(listIndex, serviceIndex);
                },
                function () {
                    UIMiniFramework.Modules.Pvr.scheduledRecording.isTriggered = false;
                    PluginInterface.Pvr.fDestroyScheduledRecord(currentRecordIndex);
                },
                false,
                true,
                false,
                60000,
                null,
                false
            );
        } else if (parseInt(isStarting, 10) == 0) {
            UIMiniFramework.Modules.Confirmations.add(
                UIMiniFramework.Translations.OSD.systemMessages.OSD_355.part1[UIMiniFramework.Globals.language] + eventName +
                UIMiniFramework.Translations.OSD.systemMessages.OSD_355.part2[UIMiniFramework.Globals.language] + serviceName +
                UIMiniFramework.Translations.OSD.systemMessages.OSD_355.part3[UIMiniFramework.Globals.language],
                null,
                function () {
                    UIMiniFramework.Modules.Pvr.scheduledRecording.isTriggered = false;
                    UIMiniFramework.Modules.ZapperWrapper.setActiveService(listIndex, serviceIndex);
                },
                function () {
                    UIMiniFramework.Modules.Pvr.scheduledRecording.isTriggered = false;
                    PluginInterface.Pvr.fDestroyScheduledRecord(currentRecordIndex);
                },
                false,
                true,
                false,
                50000,
                null,
                false
            );
        }
    },

    languageCallback: function (language) {
        for (var i in UIMiniFramework.Globals.languages.items) {
            var item = UIMiniFramework.Globals.languages.items[i];
            if (item == language) {
                PluginInterface.Setup.setOsdLanguage(language);
                UIMiniFramework.Globals.language = language;
            }
        }
    },

    streamComponentNotification: function(route, event){
        if (parseInt(event, 10) === 1) {
            if (UIMiniFramework.Config.MODULES.Teletext) {
                UIMiniFramework.Modules.Teletext.destroy();
            }
            if (UIMiniFramework.Config.MODULES.Hbbtv){
                UIMiniFramework.Modules.Hbbtv.hide();
            }
            if (UIMiniFramework.Config.MODULES.LanguageSubtitle && UIMiniFramework.Modules.LanguageSubtitle.isCreated) {
                UIMiniFramework.Modules.LanguageSubtitle.destroy();
            }
        }
    },

    // callback to execute when media player state has changed
    mediaPlayerCallback: function (state) {
        UIMiniFramework.Modules.MediaPlayer.mediaPlayerCallback(state);
    },

    pvrPlayerCallback: function (event, pData) {
        console.log('CALLBACK ' + event + ' | ' + pData);
        UIMiniFramework.Modules.MediaPlayer.pvrPlayerCallback(event, pData);
        switch (event) {
            case 12://DBUS_PVR_EVENT_PLAYBACK_POSITION
                break;
            case 11://DBUS_PVR_EVENT_PLAYBACK_STOP
                console.log('CALLBACK ' + event + ' | ' + pData);
                UIMiniFramework.Modules.Confirmations.removeByCode('message67Code');
                UIMiniFramework.Modules.Confirmations.removeByCode('message69Code');
                break;
            case 30://DBUS_PVR_EVENT_END_OF_FILE
                break;
            case 32: //DBUS_PVR_EVENT_PLAYBACK_MEDIA_EXPIRED
                UIMiniFramework.Modules.Confirmations.removeByCode('message60Code');
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_060[UIMiniFramework.Globals.language],
                    [],
                    null,
                    null,
                    false,
                    false,
                    false,
                    null,
                    null,
                    false,
                    UIMiniFramework.Translations.information[UIMiniFramework.Globals.language],
                    null,
                    null,
                    'message60Code'
                );
                break;

            case 33: //DBUS_PVR_EVENT_PLAYBACK_SPEED_NOT_ALLOWED
                UIMiniFramework.Modules.Confirmations.removeByCode('message69Code');
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_069[UIMiniFramework.Globals.language],
                    [],
                    null,
                    null,
                    false,
                    false,
                    false,
                    null,
                    null,
                    false,
                    UIMiniFramework.Translations.information[UIMiniFramework.Globals.language],
                    null,
                    null,
                    'message69Code'
                );
                break;
            case 13: //DBUS_PVR_EVENT_PLAYBACK_SPEED
                break;
            case 36: //DBUS_PVR_EVENT_PLAYBACK_EXPIRATION_DATE
                console.log('DBUS_PVR_EVENT_PLAYBACK_EXPIRATION_DATE ' + pData);
                UIMiniFramework.Modules.Confirmations.removeByCode('message67Code');
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_067[UIMiniFramework.Globals.language],
                    [],
                    null,
                    null,
                    false,
                    false,
                    false,
                    null,
                    null,
                    false,
                    UIMiniFramework.Translations.information[UIMiniFramework.Globals.language],
                    {
                        $expiration_date$: pData
                    },
                    null,
                    'message67Code'
                );
                break;
        }
    },

    fastZapperCallback: function (channelIndex, isZappOnDigit) {
        isZappOnDigit = (typeof isZappOnDigit == "undefined") ? false : isZappOnDigit;
        console.log('FAST ZAPPER CALLBACK ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++' + ' | CHANNEL INDEX: ' + channelIndex + ' | izZapOnDigit: ' + isZappOnDigit);
        //cause of the zap from these applications
        if ((UIMiniFramework.Modules.TvGuide && UIMiniFramework.Modules.TvGuide.isCreated) || (UIMiniFramework.Modules.ChannelList && UIMiniFramework.Modules.ChannelList.isCreated)) {
            return true;
        }
        UIMiniFramework.Globals.service.channelIndex = channelIndex;
        if (isZappOnDigit) {
            var lcnToIndex = PluginInterface.ServiceZapper.fGetServiceIndexByLCN(UIMiniFramework.Globals.service.activeServiceList, UIMiniFramework.Globals.service.channelIndex);
            UIMiniFramework.Globals.service.channelIndex = lcnToIndex[1];
            UIMiniFramework.Globals.service.actuallServiceList = lcnToIndex[0];
        }
        else {
            UIMiniFramework.Globals.service.actuallServiceList = null;
        }

        if (UIMiniFramework.Config.MODULES.InfoBanner){
            UIMiniFramework.Modules.InfoBanner.fastZapper(channelIndex, isZappOnDigit);
        }
    },

    zapperCallback: function (components, isError) {
        isError = typeof isError == 'undefined' ? false : isError;
        if (!isError) {
            // Zap to service has been completed successfully. => Signal is available.
            UIMiniFramework.Globals.isSignalAvailable = true;
        }
        UIMiniFramework.Globals.service.isZapped = true;
        UIMiniFramework.Globals.service.activeServiceList = PluginInterface.ServiceZapper.fGetActiveServiceList();
        UIMiniFramework.Globals.service.channelIndex = PluginInterface.ServiceZapper.fGetActiveService(UIMiniFramework.Globals.service.activeServiceList);
        UIMiniFramework.Globals.service.activeChannelIndex = UIMiniFramework.Globals.service.channelIndex;
        UIMiniFramework.Modules.Loader.toggleLoader(false);
        // because on zapper error callback signal lost event is triggered, remove it on zap if success
        UIMiniFramework.Modules.PermanentInformations.removeByCode('signalStatusCode');
        console.log('ZAPPER CALLBACK | ACTUAL SERVICE LIST = ' + UIMiniFramework.Globals.service.actuallServiceList + ' | LIST = ' + UIMiniFramework.Globals.service.activeServiceList + ' | SERVICE = ' + UIMiniFramework.Globals.service.channelIndex);

        if (UIMiniFramework.Config.MODULES.Pvr) {
            UIMiniFramework.Modules.Pvr.handleScheduledRecording(isError);
            UIMiniFramework.Modules.Pvr.handleStandbyRecordingIssue();
        }
        UIMiniFramework.Modules.UsbDevices.handleFormatPostponedIssue();
        UIMiniFramework.Modules.DummyVideoController.setVideo(UIMiniFramework.Modules.DummyVideoController.videoStreamPath + '/video_' + UIMiniFramework.Globals.service.channelIndex + '.mp4');

        // update front panel
        if (UIMiniFramework.Config.MODULES.TvGuide && !UIMiniFramework.Modules.TvGuide.isCreated || !UIMiniFramework.Modules.TvGuide) {
            UIMiniFramework.Modules.FrontPanel.update();
        }

        if (UIMiniFramework.Config.MODULES.InfoBanner){
            UIMiniFramework.Modules.InfoBanner.zapperCallback(components, isError);
        }
        if (UIMiniFramework.Config.MODULES.ChannelList){
            UIMiniFramework.Modules.ChannelList.zapperCallback(components, isError);
        }

        if (UIMiniFramework.Config.MODULES.TvGuide) {
            UIMiniFramework.Modules.TvGuide.zapperCallback(components, isError);
        }
    },

    digitZapNotificationCallback: function (status) {
        UIMiniFramework.Globals.service.isZapOnDigitInProgress = status;
    },

    refreshEpgItems: function () {
        if (UIMiniFramework.Config.MODULES.TvGuide && UIMiniFramework.Modules.Input.currentFocus == UIMiniFramework.Modules.TvGuide) {
            UIMiniFramework.Modules.TvGuide.writeEpgItems();
            UIMiniFramework.Modules.TvGuide.setSelectedItem();
            UIMiniFramework.Modules.TvGuide.updateDetails();
        }
    },

    multimediaBrowsingCallback: function () {
        if (UIMiniFramework.Config.MODULES.Multimedia && UIMiniFramework.Modules.Input.currentFocus == UIMiniFramework.Modules.Multimedia) {
            UIMiniFramework.Modules.Multimedia.multimediaBrowsingCallback();
        }
    },

    pvrCallback: function (event, data1, data2) {
        console.log(event + ' | ' + data1 + ' | ' + data2);
        event = parseInt(event, 10);
        switch (event) {
            case 31: //DBUS_PVR_EVENT_RECORD_NOT_ALLOWED
                UIMiniFramework.Modules.Confirmations.removeByCode('message61Code');
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.OSD.systemMessages.OSD_061[UIMiniFramework.Globals.language],
                    [],
                    null,
                    null,
                    false,
                    false,
                    false,
                    null,
                    null,
                    false,
                    UIMiniFramework.Translations.information[UIMiniFramework.Globals.language],
                    null,
                    null,
                    'message61Code'
                );
                break;
            default:
                break;
        }
        if (UIMiniFramework.Config.MODULES.Pvr){
            UIMiniFramework.Modules.Pvr.pvrCallback(event, data1, data2);
        }
    },

    timeshiftCallback: function(speed, event, data1, data2, data3){
        //console.log(speed + ' | ' + event + ' | ' + data1 + ' | ' + data2 + ' | ' + data3);
        switch (event) {
            case PluginInterface.Pvr.ePvrEvent.TIMESHIFT_START:
                break;
            case PluginInterface.Pvr.ePvrEvent.TIMESHIFT_STOP:
                UIMiniFramework.Modules.Confirmations.removeByCode('message69Code');
                break;
            case PluginInterface.Pvr.ePvrEvent.TIMESHIFT_POSITION:
                break;
            case PluginInterface.Pvr.ePvrEvent.TIMESHIFT_SPEED:
                break;
            case PluginInterface.Pvr.ePvrEvent.PLAYBACK_SPEED_NOT_ALLOWED:
                break;
            case PluginInterface.Pvr.ePvrEvent.LIMITED_TIMESHIFT:
                break;
            case PluginInterface.Pvr.ePvrEvent.TIMESHIFT_NOT_ALLOWED:
                break;
            case PluginInterface.Pvr.ePvrEvent.INCREASE_TIMESHIFT_NOT_ALLOWED:
                break;
        }

        if (UIMiniFramework.Config.MODULES.Timeshift){
            UIMiniFramework.Modules.Timeshift.timeshiftCallback(speed, event, data1, data2, data3);
        }
    },

    pvrBrowsingCallback: function(success){
        if (UIMiniFramework.Config.MODULES.RecordedTv){
            UIMiniFramework.Modules.RecordedTv.pvrBrowsingCallback(success);
        }
    },

    dmrCallback: function(param){
        if (UIMiniFramework.Config.MODULES.DMR){
            UIMiniFramework.Modules.DMR.dmrCallback(param);
        }
    },

    scanStatusCallback: function(event, value){
        if (UIMiniFramework.Config.MODULES.channelScan){
            UIMiniFramework.Modules.Settings.screens.channelScan.scanStatusCallback(event, value);
        }
    },

    fetchTranspondersCallback: function(event, value){
        if (UIMiniFramework.Config.MODULES.satelliteScan){
            UIMiniFramework.Modules.Settings.screens.satelliteScan.fetchTranspondersCallback(event, value);
        }
    }
};