"use strict";

ui.App.KeyHandlers = {

    record: function(){
        if (!UIMiniFramework.Config.MODULES.Pvr || !UIMiniFramework.Globals.service.isZapped || UIMiniFramework.Globals.service.isServiceLocked || UIMiniFramework.Globals.service.isServiceAgeLocked) {
            return true;
        }
        if (!UIMiniFramework.Globals.isSignalAvailable) {
            UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.noSignal[UIMiniFramework.Globals.language]);
            return true;
        }
        if (!PluginInterface.Setup.getPvrActivated()) {
            UIMiniFramework.Tools.recordingDisabledInformation();
            return true;
        }
        if (!UIMiniFramework.Modules.UsbDevices.recorder.isMounted) {
            UIMiniFramework.Modules.UsbDevices.usbNotReady();
            return true;
        }
        if (UIMiniFramework.Modules.Pvr.isRecording()) {
            UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.OSD.systemMessages.OSD_352[UIMiniFramework.Globals.language]);
            return true;
        }
        UIMiniFramework.Modules.Pvr.oneTouchRecording.create();
        UIMiniFramework.Modules.Pvr.oneTouchRecording.show();
    },

    epg: function(){
        if (UIMiniFramework.Config.MODULES.TvGuide) {
            if (!UIMiniFramework.Modules.TvGuide.isCreated) {
                UIMiniFramework.Modules.Input.exit();
                UIMiniFramework.Modules.TvGuide.create(false);
                UIMiniFramework.Modules.TvGuide.show();
            }
            else {
                UIMiniFramework.Modules.TvGuide.destroy(true);
            }
        }
    },

    ok: function(){
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            if (UIMiniFramework.Config.MODULES.ChannelList) {
                UIMiniFramework.Modules.ChannelList.create(PluginInterface.ServiceZapper.fGetActiveServiceList(), false);
                UIMiniFramework.Modules.ChannelList.show();
            }
        }
    },

    back: function(){
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            UIMiniFramework.Modules.ZapperWrapper.swapLastServices();
        }
    },

    favorite: function(){
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            if (UIMiniFramework.Config.MODULES.ChannelList) {
                UIMiniFramework.Modules.ChannelList.create(PluginInterface.ServiceZapper.serviceList.FAVORITE_LIST1, true);
                UIMiniFramework.Modules.ChannelList.show();
            }
        }
    },

    tvRadio: function(){
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            if (UIMiniFramework.Config.MODULES.ChannelList) {
                UIMiniFramework.Modules.ChannelList.create(PluginInterface.ServiceZapper.serviceList.TV_LIST, true, true);
                UIMiniFramework.Modules.ChannelList.show();
            }
        }
    },

    menu: function(){
        if (UIMiniFramework.Config.MODULES.Menu) {
            if (!UIMiniFramework.Modules.Menu.isCreated) {
                UIMiniFramework.Modules.Input.exit();
                UIMiniFramework.Modules.Menu.create(true);
            }
            else {
                UIMiniFramework.Modules.Menu.destroy(true);
            }
        }
    },

    info: function(){
        // call info banner only from live tv and the following applications
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            UIMiniFramework.Modules.Input.exit();
            if (UIMiniFramework.Config.MODULES.InfoBanner) {
                UIMiniFramework.Modules.InfoBanner.create(0, -1, -1, false, false, true);
                clearTimeout(UIMiniFramework.Modules.InfoBanner.autoCloseTimeoutId);
            }
        }
    },

    channelUp: function(){
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            UIMiniFramework.Modules.ZapperWrapper.setNextService();
        }
    },

    channelDown: function(){
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            UIMiniFramework.Modules.ZapperWrapper.setPreviousService();
        }
    },

    languageSubtitle: function(keyCode){
        if (!UIMiniFramework.Globals.service.isZapped || UIMiniFramework.Globals.service.isServiceLocked || UIMiniFramework.Globals.service.isServiceAgeLocked) {
            return true;
        }
        if (!UIMiniFramework.Globals.isSignalAvailable) {
            UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.noSignal[UIMiniFramework.Globals.language]);
            return true;
        }
        UIMiniFramework.Modules.Input.exit();
        if (UIMiniFramework.Config.MODULES.LanguageSubtitle) {
            if (!UIMiniFramework.Modules.LanguageSubtitle.isCreated) {
                UIMiniFramework.Modules.LanguageSubtitle.create(keyCode);
                UIMiniFramework.Modules.LanguageSubtitle.show();
            }
        }
    },

    mediaPlayer: function(){
        if (!UIMiniFramework.Modules.UsbDevices.multimedia.isMounted) {
            UIMiniFramework.Modules.UsbDevices.usbNotReady();
            return true;
        }
        if (UIMiniFramework.Config.MODULES.Multimedia) {
            if (!UIMiniFramework.Modules.Multimedia.isCreated) {
                UIMiniFramework.Modules.Input.exit();
                UIMiniFramework.Modules.Multimedia.create(false, UIMiniFramework.Globals.modules.multimedia.photos);
                UIMiniFramework.Modules.Multimedia.show();
            }
            else {
                UIMiniFramework.Modules.Multimedia.destroy(true);
            }
        }
    },

    scheduler: function(){
        if (UIMiniFramework.Config.MODULES.Scheduler) {
            if (!UIMiniFramework.Modules.Scheduler.isCreated) {
                UIMiniFramework.Modules.Input.exit();
                UIMiniFramework.Modules.Scheduler.create(false);
                UIMiniFramework.Modules.Scheduler.show();
            }
            else {
                UIMiniFramework.Modules.Scheduler.destroy(true);
            }
        }
    },

    pvr: function(){
        if (!PluginInterface.Setup.getPvrActivated()) {
            UIMiniFramework.Tools.recordingDisabledInformation();
            return true;
        }
        if (UIMiniFramework.Config.MODULES.RecordedTv) {
            if (!UIMiniFramework.Modules.RecordedTv.isCreated) {
                UIMiniFramework.Modules.Input.exit();
                UIMiniFramework.Modules.RecordedTv.create(false);
                UIMiniFramework.Modules.RecordedTv.show();
            }
            else {
                UIMiniFramework.Modules.RecordedTv.destroy(true);
            }
        }
    },

    teletext: function(){
        if (!UIMiniFramework.Globals.service.isZapped || UIMiniFramework.Globals.service.isServiceLocked || UIMiniFramework.Globals.service.isServiceAgeLocked) {
            return true;
        }
        if (!UIMiniFramework.Globals.isSignalAvailable) {
            UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.noSignal[UIMiniFramework.Globals.language]);
            return true;
        }
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            if (UIMiniFramework.Config.MODULES.Teletext && !UIMiniFramework.Modules.Teletext.isCreated) {
                UIMiniFramework.Modules.Teletext.create();
            }
        }
    },

    power: function(){
        if (UIMiniFramework.Globals.isStandby && keySource == UIMiniFramework.Globals.keySource.FRONT_PANEL) {
            return true;
        }

        if (UIMiniFramework.Modules.Settings.mode == UIMiniFramework.Modules.Settings.modes.fti) {
            UIMiniFramework.Globals.isStandbyTriggered = true;
            PluginInterface.Setup.fPowerOff();
        }
        else {
            if (UIMiniFramework.Config.MODULES.Timeshift && UIMiniFramework.Modules.Timeshift.isTimeshiftRecording) {
                UIMiniFramework.Modules.Timeshift.handleTimeshiftStandby();
                return true;
            }
            if (!UIMiniFramework.Globals.service.isZapped) {
                return true;
            }
            UIMiniFramework.Globals.isStandbyTriggered = true;
            PluginInterface.Setup.fPowerOff();
        }
    },

    playPause: function(){
        if (!UIMiniFramework.Globals.service.isZapped || UIMiniFramework.Globals.service.isServiceLocked || UIMiniFramework.Globals.service.isServiceAgeLocked) {
            return true;
        }
        if (!PluginInterface.Setup.getPvrActivated()) {
            UIMiniFramework.Tools.recordingDisabledInformation();
            return true;
        }
        if (!UIMiniFramework.Globals.isSignalAvailable) {
            UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.noSignal[UIMiniFramework.Globals.language]);
            return true;
        }
        if (!UIMiniFramework.Modules.UsbDevices.recorder.isMounted) {
            UIMiniFramework.Modules.UsbDevices.usbNotReady();
            return false;
        }
        // process recording lack of storage calculation;
        // 1st argument - desired duration in seconds; default 90 min for timeshift
        // 2nd argument - service type sd, hd, uhd or null if want to set service type of the currently active service
        // 3rd argument - means do not check required space (just predefined min buffer size in Mb -> UIMiniFramework.Config.recordingMinimumBufferSizeInMb)
        if (UIMiniFramework.Modules.Pvr.isRecordingLackOfStorage(90 * 60, null, false)) {
            return false;
        }
        if (UIMiniFramework.Config.MODULES.Timeshift) {
            UIMiniFramework.Modules.Input.exit();
            if (!UIMiniFramework.Modules.Timeshift.isCreated) {
                UIMiniFramework.Modules.Timeshift.create();
            }
        }
    },

    number: function(keyCode){
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            UIMiniFramework.Modules.ZapperWrapper.setServiceOnDigit(keyCode);
        }
    },

    handleKey: function(keyCode, keySource, skipSystemManager, isManuallyTriggered) {
        skipSystemManager = typeof skipSystemManager == 'undefined' ? false : skipSystemManager;
        isManuallyTriggered = typeof isManuallyTriggered == 'undefined' ? false : isManuallyTriggered;

        /* Send key code to System Manager whatever the key pressed and whatever the context - Mandatory for every key pressed - Keep this call as 1st action in this function */
        if (!skipSystemManager) {
            PluginInterface.Setup.fSendKeyToSystemManager(UIMiniFramework.SysKeys.getSysKeyCode(keyCode), keySource);
        }

        if (UIMiniFramework.Modules.Input.isLocked && !isManuallyTriggered) {
            return true;
        }

        // example of test suit usage
        //if(keyCode == UIMiniFramework.Keys.OK) {
        //    var ts = new UIMiniFramework.Modules.TestSuite("Auto press RIGHT", UIMiniFramework.Keys.Right, 0.1, 3, 10);
        //    return true;
        //}

        // confirm and close specific entitlement messages on OK, Exit or Back key if there is no app running
        if (!UIMiniFramework.Modules.Input.currentFocus && !UIMiniFramework.Globals.isStandbyTriggered) {
            if (UIMiniFramework.Modules.PermanentInformations.handleKeyPress(keyCode)) {
                return true;
            }
        }

        if (UIMiniFramework.Modules.Input.currentFocus && UIMiniFramework.Modules.Input.currentFocus.handleKeyPress(keyCode)) {
            UIMiniFramework.Modules.PermanentInformations.togglePermanentInfo();
            if (UIMiniFramework.Config.MODULES.LanguageSubtitle){
                UIMiniFramework.Modules.LanguageSubtitle.toggleSubtitle();
            }
            if (UIMiniFramework.Modules.Input.currentFocus != UIMiniFramework.Modules.PinEntry) {
                UIMiniFramework.Modules.PinEntry.toggle(keyCode);
            }
            // if any app is active
            if (UIMiniFramework.Modules.Input.currentFocus) {
                if (UIMiniFramework.Config.MODULES.Hbbtv) {
                    UIMiniFramework.Modules.Hbbtv.hide();
                }
            }
            return true;
        }

        // keyhandling
        switch (keyCode) {
            case UIMiniFramework.Keys.OK:
                this.ok();
                break;
            case UIMiniFramework.Keys.Favorite:
                this.favorite();
                break;
            case UIMiniFramework.Keys.TvRadio:
                this.tvRadio();
                break;
            case UIMiniFramework.Keys.Menu:
                this.menu();
                break;
            case UIMiniFramework.Keys.Info:
                this.info();
                break;
            case UIMiniFramework.Keys.Record:
                if (this.record()){
                    return true;
                }
                break;
            case UIMiniFramework.Keys.Stop:
                UIMiniFramework.Modules.Pvr.stopRecording();
                break;
            case UIMiniFramework.Keys.Epg:
                this.epg();
                break;
            case UIMiniFramework.Keys.ChannelUp:
                this.channelUp();
                break;
            case UIMiniFramework.Keys.ChannelDown:
                this.channelDown();
                break;
            case UIMiniFramework.Keys.VolumeUp:
                UIMiniFramework.Modules.VolumeBar.updateVolume(1);
                break;
            case UIMiniFramework.Keys.VolumeDown:
                UIMiniFramework.Modules.VolumeBar.updateVolume(-1);
                break;
            case UIMiniFramework.Keys.Mute:
                UIMiniFramework.Modules.VolumeBar.toggleMute();
                break;
            case UIMiniFramework.Keys.Red:
                if (UIMiniFramework.Globals.service.isServiceLocked || UIMiniFramework.Globals.service.isServiceAgeLocked) {
                    return true;
                }
                break;
            case UIMiniFramework.Keys.Language:
            case UIMiniFramework.Keys.Subtitle:
                if (this.languageSubtitle(keyCode)){
                    return true;
                }
                break;
            case UIMiniFramework.Keys.MediaPlayer:
                if (this.mediaPlayer()){
                    return true;
                }
                break;
            case UIMiniFramework.Keys.SCHED:
                this.scheduler();
                break;
            case UIMiniFramework.Keys.PVR:
                if (this.pvr()){
                    return true;
                }
                break;
            case UIMiniFramework.Keys.RES:
                break;
            case UIMiniFramework.Keys.Exit:
                UIMiniFramework.Modules.Input.exit();
                break;
            case UIMiniFramework.Keys.Back:
                this.back();
                break;
            case UIMiniFramework.Keys.Teletext:
                if (this.teletext()){
                    return true;
                }
                break;
            case UIMiniFramework.Keys.Power:
                if (this.power()){
                    return true;
                }
                break;
            case UIMiniFramework.Keys.PlayPause:
                if (this.playPause()){
                    return true;
                }
                break;
            case UIMiniFramework.Keys.Num0:
            case UIMiniFramework.Keys.Num1:
            case UIMiniFramework.Keys.Num2:
            case UIMiniFramework.Keys.Num3:
            case UIMiniFramework.Keys.Num4:
            case UIMiniFramework.Keys.Num5:
            case UIMiniFramework.Keys.Num6:
            case UIMiniFramework.Keys.Num7:
            case UIMiniFramework.Keys.Num8:
            case UIMiniFramework.Keys.Num9:
                this.number(keyCode - UIMiniFramework.Keys.Num0);
                break;
        }

        // if any app is active
        if (UIMiniFramework.Modules.Input.currentFocus) {
            if (UIMiniFramework.Config.MODULES.Hbbtv) {
                UIMiniFramework.Modules.Hbbtv.hide();
            }
        }
        else {
            // on exit we call restartHbbtw which shows hbbtv banner so no need to show it again
            if (keyCode != UIMiniFramework.Keys.Exit) {
                if (UIMiniFramework.Config.MODULES.Hbbtv) {
                    UIMiniFramework.Modules.Hbbtv.show();
                }
            }
        }
        UIMiniFramework.Modules.PermanentInformations.togglePermanentInfo();
        if (UIMiniFramework.Config.MODULES.LanguageSubtitle && !UIMiniFramework.Modules.LanguageSubtitle.isCreated) {
            UIMiniFramework.Modules.LanguageSubtitle.toggleSubtitle();
        }
        UIMiniFramework.Modules.PinEntry.toggle(keyCode);
        return true;
    }
};