"use strict";

UIMiniFramework.Modules.PermanentInformations = {
    el: null,
    // css selector for wrapper where it'll be appended
    wrapperSelector: UIMiniFramework.Config.bodySelector,
    // tells if component is created, but not shown
    isCreated: false,
    // tells if component is shown, must be created
    isShown: false,
    // notifications
    items: [],
    // count of total informations
    totalCount: 0,
    updateStatus: {
        initialized: 0,
        added: 1,
        removed: 2
    },
    code: null,

    // namespace placeholders for custom permanent informations
    customPermanentInformations: null,
    permanentMessageCodes: {},

    create: function () {
        if (UIMiniFramework.Config.MODULES.Teletext) {
            UIMiniFramework.Modules.Teletext.hide();
        }
        this.init();
        this.render();
        this.hide();
        this.addToDOM();
        this.isCreated = true;
        this.isShown = false;
        this.customPermanentInformations.toggleExternalComponents(true);
    },

    destroy: function () {
        this.removeFromDOM();
        this.isCreated = false;
        this.isShown = false;
        this.items.length = 0;
        this.caller = null;
        this.customPermanentInformations.toggleExternalComponents(false);
        this.customPermanentInformations = null;
        if (UIMiniFramework.Config.MODULES.Teletext) {
            UIMiniFramework.Modules.Teletext.show();
        }
        if (UIMiniFramework.Config.MODULES.Hbbtv) {
            UIMiniFramework.Modules.Hbbtv.show();
        }
    },

    init: function () {
        this.customPermanentInformations = ui.Mappings.CustomPermanentInformations;
    },

    render: function () {
        this.el = $(UIMiniFramework.Templates.permanentInformations.wrapper);
    },

    addToDOM: function () {
        $(this.wrapperSelector).append(this.el);
    },

    removeFromDOM: function () {
        if (this.isCreated) {
            try {
                this.el.remove();
                this.el = null;
            }
            catch (error) {
            }
        }
    },

    show: function () {
        if (!UIMiniFramework.Modules.Loader.isCreated) {
            this.el.removeClass('invisible');
        }
        this.isShown = true;
    },

    hide: function () {
        this.el.addClass('invisible');
        this.isShown = false;
    },

    add: function (data, replacableData, isTopPriority, isClosable, messageCode, title) {
        if (!this.isCreated) {
            this.create();
        }
        messageCode = messageCode || '';
        var information = new UIMiniFramework.CustomComponents.permanentInformationItem(data, replacableData, isTopPriority, isClosable, messageCode, title);
        information.init();
        if (messageCode !== '') {
            this.permanentMessageCodes[messageCode] = information.code;
        }
        this.code = information.code;
        this.onAddEvent(messageCode);
        this.togglePermanentInfo();
        return this.code;
    },

    remove: function () {
        if (this.items.length > 0) {
            var information = this.items.shift();
            var response = information.stop();
            this.onRemoveEvent(response);
        }
    },

    removeByCode: function (messageName) {
        var code = this.permanentMessageCodes[messageName] || null;
        if (code == null) {
            return false;
        }
        if (this.items.length > 0) {
            for (var i in this.items) {
                var information = this.items[i];
                if (information.code === code) {
                    this.items.splice(i, 1);
                    var response = information.stop();
                    this.permanentMessageCodes[messageName] = null;
                    this.onRemoveEvent(response);
                    return true;
                }
            }
        }
    },

    updated: function (updateStatus) {
        switch (updateStatus) {
            case this.updateStatus.initialized:
                ++this.totalCount;
                break;
            case this.updateStatus.added:
                break;
            case this.updateStatus.removed:
                --this.totalCount;
                if (this.totalCount == 0) {
                    this.destroy();
                }
                break;
        }
    },

    onAddEvent: function (messageCode) {
        if (typeof messageCode == 'undefined' || !messageCode) {
            return;
        }

        try {
            this.customPermanentInformations[messageCode].create.call(this.customPermanentInformations[messageCode], messageCode);
        }
        catch (error) {
        }
    },

    onRemoveEvent: function (messageCode) {
        if (typeof messageCode == 'undefined' || !messageCode) {
            return;
        }

        try {
            this.customPermanentInformations[messageCode].destroy.call(this.customPermanentInformations[messageCode], messageCode);
        }
        catch (error) {
        }
    },

    togglePermanentInfo: function () {
        if (!this.isCreated) {
            return false;
        }
        // check if any app is active
        if (UIMiniFramework.Modules.Input.currentFocus != null) {
            if (this.isShown) {
                this.hide();
            }
            return false;
        }

        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            if (!this.isShown) {
                this.show();
            }
            return false;
        }
    },

    handleKeyPress: function (keyCode) {
        if (!this.isCreated) {
            return false;
        }
        try {
            var messageCode = this.items[0].messageCode; // 0 index means top most item - that is focused
            var response = this.customPermanentInformations[messageCode].handleKeyPress.call(this.customPermanentInformations[messageCode], keyCode);
            return response;
        }
        catch (error) {
            return false;
        }
    }
};