"use strict";

UIMiniFramework.Modules.PinEntry = {
    el: null,
    // css selector for wrapper where it'll be appended
    wrapperSelector: UIMiniFramework.Config.uiAppWrapperSelector,
    // tells if component is created, but not shown
    isCreated: false,
    // tells if component is shown, must be created
    isShown: false,
    caller: null,
    lockType: null,
    pinSuccessCallback: null,

    pinBlocked: {
        status: false,
        timerId: null,
        timerCount: UIMiniFramework.Config.smartCardPinBlockedSeconds,
        init: function () {
            clearInterval(UIMiniFramework.Modules.PinEntry.pinBlocked.timerId);
            this.timerId = setInterval(function () {
                --UIMiniFramework.Modules.PinEntry.pinBlocked.timerCount;
                if (UIMiniFramework.Modules.PinEntry.pinBlocked.timerCount == 0) {
                    UIMiniFramework.Globals.smartCardState = 1;
                    UIMiniFramework.Modules.PinEntry.pinBlocked.timerCount = UIMiniFramework.Config.smartCardPinBlockedSeconds;
                    clearInterval(UIMiniFramework.Modules.PinEntry.pinBlocked.timerId);
                    if (UIMiniFramework.Modules.PinEntry.isCreated && UIMiniFramework.Modules.PinEntry.pinBlocked.status) {
                        UIMiniFramework.Modules.PinEntry.destroy(PluginInterface.ParentalControl.eLockType.AGE);
                    }
                    UIMiniFramework.Modules.PinEntry.pinBlocked.status = false;
                }
                UIMiniFramework.Modules.PinEntry.pinBlocked.displayTimer();
            }, 1000);
        },

        displayTimer: function () {
            if (UIMiniFramework.Modules.PinEntry.isCreated && UIMiniFramework.Modules.PinEntry.pinBlocked.status) {
                if (UIMiniFramework.Modules.PinEntry.pinBlocked.timerCount > 60) {
                    $('.pinEntry.pinBlocked .footer').text(parseInt(UIMiniFramework.Modules.PinEntry.pinBlocked.timerCount / 60) + ' min ' + UIMiniFramework.Modules.PinEntry.pinBlocked.timerCount % 60 + ' ' + UIMiniFramework.Translations.seconds[UIMiniFramework.Globals.language]);
                }
                else {
                    $('.pinEntry.pinBlocked .footer').text(UIMiniFramework.Modules.PinEntry.pinBlocked.timerCount % 60 + ' ' + UIMiniFramework.Translations.seconds[UIMiniFramework.Globals.language]);
                }
            }
        }
    },

    display: function (pinSuccessCallback, lockType, title) {
        if (this.isCreated) {
            return false;
        }
        console.log('DISPLAY PIN ENTRY | LOCK TYPE = ' + lockType);
        this.lockType = lockType;
        this.pinSuccessCallback = pinSuccessCallback;
        this.title = typeof title == 'undefined' ? UIMiniFramework.Translations.enterPin[UIMiniFramework.Globals.language] : title;
        UIMiniFramework.Globals.service.isServiceLocked = (this.lockType == PluginInterface.ParentalControl.eLockType.CHANNEL);
        UIMiniFramework.Globals.service.isServiceAgeLocked = (this.lockType == PluginInterface.ParentalControl.eLockType.AGE);
        ui.App.Tools.toggleLockedOverlay(false);
        if ((UIMiniFramework.Modules.Input.currentFocus == null && !ui.App.CustomPermanentInformations.closableEntitlementMessage.isCreated)
            || (UIMiniFramework.Config.MODULES.ChannelList && UIMiniFramework.Modules.Input.currentFocus == UIMiniFramework.Modules.ChannelList)
            || (UIMiniFramework.Config.MODULES.TvGuide && UIMiniFramework.Modules.Input.currentFocus == UIMiniFramework.Modules.TvGuide)
            || (UIMiniFramework.Config.MODULES.InfoBanner && UIMiniFramework.Modules.Input.currentFocus == UIMiniFramework.Modules.InfoBanner)
            || (UIMiniFramework.Config.MODULES.RecordedTv && UIMiniFramework.Modules.Input.currentFocus == UIMiniFramework.Modules.RecordedTv)
        ) {
            this.create();
            this.show();
        }
    },

    create: function () {
        this.isCreated = true;
        this.pinBlocked.status = ((this.lockType == PluginInterface.ParentalControl.eLockType.AGE || this.lockType == PluginInterface.ParentalControl.eLockType.PVR_AGE) && UIMiniFramework.Globals.smartCardState == 4);
        this.init();
        this.render();
        this.hide();
        this.addToDOM();
        this.updateLabels();
        this.focus();
        this.isShown = false;
        this.pinBlocked.displayTimer();
        UIMiniFramework.Modules.PermanentInformations.togglePermanentInfo();
    },

    destroy: function (lockType, forceDestroy) {
        forceDestroy = typeof forceDestroy == 'undefined' ? false : forceDestroy;
        if (!forceDestroy) {
            if (UIMiniFramework.Globals.service.isServiceLocked && parseInt(lockType, 10) == PluginInterface.ParentalControl.eLockType.AGE) {
                return;
            }
            if (UIMiniFramework.Globals.service.isServiceAgeLocked && parseInt(lockType, 10) == PluginInterface.ParentalControl.eLockType.CHANNEL) {
                return;
            }
        }
        if (!this.isCreated) {
            return;
        }
        console.log('pin entry DESTROY');
        this.isCreated = false;
        this.removeFromDOM();
        this.isShown = false;
        this.blur();
        this.caller = null;
        UIMiniFramework.Modules.PermanentInformations.togglePermanentInfo();
        setTimeout(ui.App.Tools.toggleLockedOverlay.bind(ui.App.Tools), 700);
    },

    init: function(){
        this.components = new ui.Mappings.PinEntryItems(this);
    },

    render: function () {
        this.el = $(UIMiniFramework.Templates.genericScreen);
        this.el.addClass('pinEntry');
        switch (this.lockType) {
            case PluginInterface.ParentalControl.eLockType.AGE:
                this.el.addClass('ageLocked');
                this.el.toggleClass('pinBlocked', this.pinBlocked.status);
                break;
        }
        if (this.components && this.components.renderComponents) {
            this.components.renderComponents();
        }
    },

    updateLabels: function () {
        if (this.components && this.components.updateComponentLabels) {
            this.components.updateComponentLabels();
        }
    },

    focus: function () {
        this.caller = UIMiniFramework.Modules.Input.currentFocus;
        UIMiniFramework.Modules.Input.currentFocus = this;
    },

    blur: function () {
        if (this.caller && this.caller.isCreated) {
            UIMiniFramework.Modules.Input.currentFocus = this.caller;
        }
        else {
            UIMiniFramework.Modules.Input.resetFocus();
        }
    },

    addToDOM: function () {
        $(this.wrapperSelector).append(this.el);
    },

    removeFromDOM: function () {
        this.el.remove();
        this.el = null;
    },

    show: function () {
        if (!UIMiniFramework.Modules.Loader.isCreated) {
            this.el.removeClass('invisible');
        }
        this.isShown = true;
    },

    hide: function () {
        this.el.addClass('invisible');
        this.isShown = false;
    },

    channelUpDown: function (keyCode) {
        UIMiniFramework.Globals.service.isServiceLocked = false;
        UIMiniFramework.Globals.service.isServiceAgeLocked = false;
        if (UIMiniFramework.Config.MODULES.RadioPlayingOverlay) {
            UIMiniFramework.Modules.RadioPlayingOverlay.destroy();
        }
        this.destroy();
        switch (keyCode){
            case UIMiniFramework.Keys.ChannelUp:
                UIMiniFramework.Modules.ZapperWrapper.setNextService();
                break;
            case UIMiniFramework.Keys.ChannelDown:
                UIMiniFramework.Modules.ZapperWrapper.setPreviousService();
                break;
        }
    },

    getAgeLockMessage: function () {
        switch (parseInt(UIMiniFramework.Globals.smartCardState)) {
            case 1: // please enter pin code
                return UIMiniFramework.Translations.OSD.systemMessages.OSD_003[UIMiniFramework.Globals.language];
            case 2: // incorrect pin first time
                return UIMiniFramework.Translations.OSD.systemMessages.OSD_004[UIMiniFramework.Globals.language];
            case 3:// incorrect pin second time
                return UIMiniFramework.Translations.OSD.systemMessages.OSD_005[UIMiniFramework.Globals.language];
            case 4:// incorrect pin third time - disable for 10 minutes
                return UIMiniFramework.Translations.OSD.systemMessages.OSD_006[UIMiniFramework.Globals.language];
        }
    },

    toggle: function (keyCode) {
        keyCode == typeof keyCode == 'undefined' ? -1 : keyCode;
        if (keyCode == UIMiniFramework.Keys.Exit) {
            UIMiniFramework.Modules.PinEntry.destroy(-1, true);
            return;
        }

        switch (keyCode) {
            case UIMiniFramework.Keys.Mute:
            case UIMiniFramework.Keys.VolumeUp:
            case UIMiniFramework.Keys.VolumeDown:
                return;
        }

        if (UIMiniFramework.Globals.service.isServiceLocked) {
            if (UIMiniFramework.Modules.Input.currentFocus == null) {
                this.display(null, PluginInterface.ParentalControl.eLockType.CHANNEL, UIMiniFramework.Translations.enterPin[UIMiniFramework.Globals.language]);
            }
            else {
                if (UIMiniFramework.Modules.Input.currentFocus == UIMiniFramework.Modules.PinEntry
                    && (UIMiniFramework.Config.MODULES.ChannelList && UIMiniFramework.Modules.PinEntry.caller != UIMiniFramework.Modules.ChannelList)
                    && (UIMiniFramework.Config.MODULES.InfoBanner && UIMiniFramework.Modules.PinEntry.caller != UIMiniFramework.Modules.InfoBanner)
                ) {
                    this.destroy(PluginInterface.ParentalControl.eLockType.CHANNEL);
                }
            }
        }
        if (UIMiniFramework.Globals.service.isServiceAgeLocked) {
            if (UIMiniFramework.Modules.Input.currentFocus == null) {
                if (!ui.App.CustomPermanentInformations.smartCardExtracted.isCreated || !ui.App.CustomPermanentInformations.smartCardCode.isCreated) {
                    var message = this.getAgeLockMessage();
                    this.display(null, PluginInterface.ParentalControl.eLockType.AGE, message);
                }
            }
            else {
                if (UIMiniFramework.Modules.Input.currentFocus == this) {
                    if (UIMiniFramework.Config.MODULES.ChannelList && this.caller != UIMiniFramework.Modules.ChannelList) {
                        this.destroy(PluginInterface.ParentalControl.eLockType.AGE);
                    }
                    if (UIMiniFramework.Config.MODULES.InfoBanner && this.caller != UIMiniFramework.Modules.InfoBanner){
                        this.destroy(PluginInterface.ParentalControl.eLockType.AGE);
                    }
                }
            }
        }
    },

    ok: function () {
        this.destroy();
        if (UIMiniFramework.Modules.Input.currentFocus == null) {
            // propagate key if no focus to open channel list
            return false;
        }
        // is something in the focus do not propagate key just close pin entry
        return true;
    },

    handleKeyPress: function (keyCode) {
        switch (keyCode) {
            case UIMiniFramework.Keys.OK:
                return this.ok();
            case UIMiniFramework.Keys.Back:
                this.destroy();
                return true;
            case UIMiniFramework.Keys.Exit:
                return true;
            case UIMiniFramework.Keys.Num0:
            case UIMiniFramework.Keys.Num1:
            case UIMiniFramework.Keys.Num2:
            case UIMiniFramework.Keys.Num3:
            case UIMiniFramework.Keys.Num4:
            case UIMiniFramework.Keys.Num5:
            case UIMiniFramework.Keys.Num6:
            case UIMiniFramework.Keys.Num7:
            case UIMiniFramework.Keys.Num8:
            case UIMiniFramework.Keys.Num9:
                if (!this.pinBlocked.status) {
                    this.components.enterNum(keyCode - UIMiniFramework.Keys.Num0);
                }
                return true;
            case UIMiniFramework.Keys.ChannelUp:
            case UIMiniFramework.Keys.ChannelDown:
                this.channelUpDown(keyCode);
                return true;
            case UIMiniFramework.Keys.VolumeUp:
            case UIMiniFramework.Keys.VolumeDown:
            case UIMiniFramework.Keys.Mute:
            case UIMiniFramework.Keys.PlayPause:
            case UIMiniFramework.Keys.Record:
            case UIMiniFramework.Keys.Red:
            case UIMiniFramework.Keys.Teletext:
            case UIMiniFramework.Keys.Language:
            case UIMiniFramework.Keys.Subtitle:
                return false;
            default:
                this.destroy();
                return false;
        }
    }
};