"use strict";

UIMiniFramework.Modules.TvGuide = {
    GUID: UIMiniFramework.Tools.getGUID(),
    // EPG DOM node
    el: null,
    // css selector for tv guide wrapper where it'll be appended
    wrapperSelector: UIMiniFramework.Config.uiAppWrapperSelector,
    // tells if tv guide is created, but not shown
    isCreated: false,
    // tells if tv guide is shown, must be created
    isShown: false,
    // items: array of objects. Of length totalVisibleItems. These are demo items and should be updated dynamically
    items: [],
    // total visible tv guide items in the list
    totalVisibleItems: 7,
    // total tv guide items (variable value)
    totalItems: 0,
    // selected item index used for vertical shifting
    selectedItemIndex: 0,
    // current tv guide item position in DOM channel item list
    currentItemPositionIndex: 0,
    // selected item index used for horizontal shifting
    selectedHorizontalItemIndex: 0,
    // epg event counts, each array member represents one epg row
    horizontalItemsCounts: [],
    // starting indexes of shown epg events
    horizontalStartItemIndexes: [],
    // service list type
    listType: 0,
    // window for collecting epg events
    durationWindow: 6,
    // difference in days between current date and selected event start date
    daysDiff: 0,
    // how many days in the future we can explore. Default 7 days
    maxDaysInFuture: 7,
    //epg clock interval id
    clockIntervalId: null,
    clockRefreshInterval: 30000,
    // channel lists
    channelLists: [],
    direction: 1,
    // should we return to menu after exiting tv guide app
    returnToMenu: false,
    scaleLiveRouteTimeoutId: null,
    // timeline component properties
    timeLineComponent: {
        startSeconds: 0,
        startString: '',
        endString: '',
        nowMilliseconds: 0,
        startLimitSeconds: 0,
        endLimitSeconds: 0,
        daysDiff: 0,
        // time line range in minutes
        timeLineRange: 125,
        // time line step in minutes
        timeLineStep: 30,
        // one minute in pixels on time line
        oneMinutePixels: 0,
        // horizontal gap between two epg events in pixels
        horizontalEpgEventsGap: 3,
        // number of time range marks
        timeRangeMarksCount: 5,
        // time line timeout in seconds. Default 30 seconds
        timeOverlayInterval: 30000,
        timeOverlayIntervalId: null,
        timeOverlayWidth: 0,
        // when moving left/right in epg list, hours to move backward/forward
        hours2ShiftOnePage: 2,
        // when moving left/right with -+24h key in epg list, hours to move backward/forward
        hours2Shift24: 24
    },

    /**
     * render channel list
     */
    render: function () {
        // create wrapper DOM element
        this.el = $(UIMiniFramework.Templates.tvGuide.wrapper);
        var header = $(UIMiniFramework.Templates.tvGuide.header);
        var list = $(UIMiniFramework.Templates.tvGuide.list);
        var epg = $(UIMiniFramework.Templates.tvGuide.epg);
        for (var i = 0; i < this.totalVisibleItems; i++) {
            var item = $(UIMiniFramework.Templates.tvGuide.item);
            var epgRow = $(UIMiniFramework.Templates.tvGuide.epgRow);
            item.addClass('item' + i);
            list.append(item);
            epg.append(epgRow);
        }
        var timeLine = $(UIMiniFramework.Templates.tvGuide.timeLine);
        var details = $(UIMiniFramework.Templates.tvGuide.details);
        var footer = $(UIMiniFramework.Templates.tvGuide.footer);
        this.el.append(header).append(list).append(epg).append(timeLine).append(details).append(footer);
    },

    initEpgParameters: function () {
        this.listType = PluginInterface.ServiceZapper.fGetActiveServiceList();
        this.timeLineComponent.oneMinutePixels = $('.timeItem1').css('left').slice(0, -2) / this.timeLineComponent.timeLineStep;
    },

    updateLabels: function () {
        this.el.find('.recordWrapper .label').text(UIMiniFramework.Translations.rec[UIMiniFramework.Globals.language]);
        this.el.find('.remindWrapper .label').text(UIMiniFramework.Translations.remind[UIMiniFramework.Globals.language]);
        this.el.find('.filterWrapper .label').text(UIMiniFramework.Translations.find[UIMiniFramework.Globals.language]);
        this.el.find('.infoWrapper .label').text(UIMiniFramework.Translations.info[UIMiniFramework.Globals.language]);
        this.el.find('.detailsWrapper .typeWrapper .label').text(UIMiniFramework.Translations.type[UIMiniFramework.Globals.language] + ':');
        this.el.find('.detailsWrapper .ageWrapper .label').text(UIMiniFramework.Translations.age[UIMiniFramework.Globals.language] + ':');
    },

    updateItems: function () {
        PluginInterface.ServiceZapper.fSetActiveServiceList(this.listType);
        this.totalItems = PluginInterface.ServiceZapper.fGetNumberOfServices(this.listType);

        this.selectedItemIndex = PluginInterface.ServiceZapper.fGetActiveService(this.listType);
        this.selectedItemIndex = this.selectedItemIndex == -1 || (UIMiniFramework.Globals.service.actuallServiceList != null && UIMiniFramework.Globals.service.actuallServiceList != this.listType) ? 0 : this.selectedItemIndex;
        var tempSelectedItemIndex = 0;
        if (this.totalItems > this.totalVisibleItems) {
            tempSelectedItemIndex = this.selectedItemIndex;
        }
        else {
            this.currentItemPositionIndex = this.selectedItemIndex;
        }

        var max = this.totalItems > this.totalVisibleItems ? this.totalVisibleItems : this.totalItems;
        this.items = [];
        for (var i = 0; i < max; i++) {
            this.items[i] = PluginInterface.ServiceZapper.fGetServiceData(this.listType, (tempSelectedItemIndex + i) % this.totalItems);
            this.items[i].realIndex = (tempSelectedItemIndex + i) % this.totalItems;
        }
        this.createEpgWindow();
    },

    /**
     * create tv guide screen
     */
    create: function (returnToMenu) {
        if (UIMiniFramework.Config.MODULES.Hbbtv) {
            UIMiniFramework.Modules.Hbbtv.freeze();
        }
        this.setChannelLists();
        this.render();
        this.updateLabels();
        this.hide();
        this.addToDOM();
        this.show();
        this.initEpgParameters();
        this.initializeTimeRangeComponent();
        this.initializeEpgClock();
        this.updateItems();
        this.writeItems();
        this.writeEpgItems();
        this.updateDetails();
        this.updateChannelList();
        this.setSelectedItem();
        this.focus();
        this.isCreated = true;
        this.isShown = true;
        this.returnToMenu = (typeof returnToMenu == 'undefined') ? true : returnToMenu;
        UIMiniFramework.Modules.FrontPanel.display('EPG');
        PluginInterface.Epg.fUpdateEPGScreen();
        this.scaleLiveRoot(350);
    },

    /**
     * destroy tv guide screen and reset all necessary states
     */
    destroy: function (exit) {
        clearInterval(this.scaleLiveRouteTimeoutId);
        PluginInterface.Epg.fScaleLiveRoute(0, 0, 0, 1280, 720);
        this.blur();
        this.removeFromDOM();
        this.el = null;
        this.isCreated = false;
        this.isShown = false;
        this.direction = 1;
        this.selectedItemIndex = 0;
        this.currentItemPositionIndex = 0;
        this.selectedHorizontalItemIndex = 0;
        this.horizontalItemsCounts.length = 0;
        this.horizontalStartItemIndexes.length = 0;
        clearInterval(this.timeLineComponent.timeOverlayIntervalId);
        clearInterval(this.clockIntervalId);
        this.timeLineComponent.timeOverlayWidth = 0;
        this.channelLists = [];
        if (UIMiniFramework.Config.MODULES.Menu) {
            UIMiniFramework.Modules.Menu.toggle(exit, this.returnToMenu);
        }
        if (UIMiniFramework.Config.MODULES.Hbbtv) {
            UIMiniFramework.Modules.Hbbtv.unFreeze();
        }
    },

    /**
     * focus tv guide so it can handle keypress events
     */
    focus: function () {
        UIMiniFramework.Modules.Input.currentFocus = this;
    },

    // blur this screen
    blur: function () {
        UIMiniFramework.Modules.Input.resetFocus();
    },

    /**
     * add tv guide to DOM
     */
    addToDOM: function () {
        $(this.wrapperSelector).append(this.el);
    },

    /**
     * remove tv guide from DOM
     */
    removeFromDOM: function () {
        this.el.remove();
    },

    /**
     * show tv guide
     */
    show: function () {
        if (!this.isShown) {
            this.el.removeClass('invisible');
        }
    },

    /**
     * hide tv guide
     */
    hide: function () {
        this.el.addClass('invisible');
        this.isShown = false;
    },

    setChannelLists: function () {
        this.channelLists = new ui.Mappings.ChannelLists().horizontalItems;
    },

    scaleLiveRoot: function (delay) {
        this.scaleLiveRouteTimeoutId = setTimeout(function () {
            PluginInterface.Epg.fScaleLiveRoute(0, 89, 465/*720-85-170*/, 302, 170);
        }, delay);
    },

    shiftVertical: function (direction) {
        if (this.totalItems <= 0) {
            return false;
        }
        //get event from previous channel, currentItemPositionIndex not yet changed
        var event = PluginInterface.Epg.fGetEpgScheduledData(this.items[this.currentItemPositionIndex].index, this.horizontalStartItemIndexes[this.currentItemPositionIndex] + this.selectedHorizontalItemIndex);

        this.setSelectedItemIndex(direction);
        var itemsChanged = this.setCurrentItemPosition(direction);

        //get index of event on current channel that matches time from previous channel event, currentItemPositionIndex has changed
        var ic = PluginInterface.Epg.fGetEpgWindow(this.items[this.currentItemPositionIndex].index, event.startdt, event.enddt).split("|");
        var index = parseInt(ic[0], 10);

        // update selected horizontal item index
        if (this.horizontalStartItemIndexes[this.currentItemPositionIndex] > index) {
            this.selectedHorizontalItemIndex = 0;
        }
        else {
            this.selectedHorizontalItemIndex = index - this.horizontalStartItemIndexes[this.currentItemPositionIndex];
        }

        if (itemsChanged) {
            if (this.totalItems > this.totalVisibleItems) {
                this.shiftItems(direction);
                this.writeItems();
                this.shiftVerticalEpgItem(direction);
            }
        }
        else {
            if (this.totalItems <= this.totalVisibleItems) {
                this.currentItemPositionIndex = this.selectedItemIndex;
            }
            this.setSelectedItem();
        }

        this.updateDetails();
    },

    shiftHorizontal: function (direction) {
        if (this.totalItems <= 0) {
            return false;
        }
        this.direction = direction;
        // disable shifting in the past and the future
        if (direction < 0) {
            if (this.selectedHorizontalItemIndex == 0 && this.timeLineComponent.startLimitSeconds == this.timeLineComponent.startSeconds) {
                return false;
            }
        }
        else {
            if (this.selectedHorizontalItemIndex == (this.horizontalItemsCounts[this.currentItemPositionIndex] - 1) || this.horizontalItemsCounts[this.currentItemPositionIndex] == 0) {
                //console.log('DAYS DIFFERENCE: ' + this.timeLineComponent.daysDiff);
                //if (this.timeLineComponent.daysDiff > this.maxDaysInFuture) {
                if ((this.timeLineComponent.startSeconds + this.timeLineComponent.hours2ShiftOnePage * 3600) >= (this.timeLineComponent.startLimitSeconds + this.maxDaysInFuture * 24 * 3600)) {
                    return false;
                }
            }
        }

        if (this.setSelectedHorizontalItemIndex(direction)) {
            this.setSelectedHorizontalItem();
        }
        else {
            this.shiftHorizontalEpgItem(direction, this.timeLineComponent.hours2ShiftOnePage);
        }
        this.setTimeOverlayWidth();
        this.updateDetails();
    },

    setSelectedHorizontalItemIndex: function (direction) {
        if (this.selectedHorizontalItemIndex + direction < 0 || this.selectedHorizontalItemIndex + direction > this.horizontalItemsCounts[this.currentItemPositionIndex] - 1) {
            return false;
        }
        else {
            this.selectedHorizontalItemIndex += direction;
            return true;
        }
    },

    setSelectedItemIndex: function (direction) {
        if (this.selectedItemIndex + direction > this.totalItems - 1) {
            this.selectedItemIndex = 0;
        }
        else if (this.selectedItemIndex + direction < 0) {
            this.selectedItemIndex = this.totalItems - 1;
        }
        else {
            this.selectedItemIndex = this.selectedItemIndex + direction;
        }
    },

    setCurrentItemPosition: function (direction) {
        //in case we shift to the item that is not in the visible list, tell items are changed
        // we'r not traversing visible items but has to fetch new one
        if (this.totalItems > this.totalVisibleItems && (this.currentItemPositionIndex + direction > this.totalVisibleItems - 1 || this.currentItemPositionIndex + direction < 0)) {
            return true;
        }
        if (this.totalItems <= this.totalVisibleItems) {
            this.currentItemPositionIndex = this.selectedItemIndex;
        }
        else {
            this.currentItemPositionIndex = (this.currentItemPositionIndex + direction) % this.totalItems;
        }
        return false;
    },

    setSelectedItem: function () {
        $('.tvGuideItem.selected, .epgRow.selected, .epgItem.selected').removeClass('selected');
        $('.tvGuideItem:nth-child(' + (this.currentItemPositionIndex + 1) + ')').addClass('selected');
        $('.epgRow:nth-child(' + (this.currentItemPositionIndex + 1) + ')').addClass('selected');
        $('.epgRow:nth-child(' + (this.currentItemPositionIndex + 1) + ')' + ' .epgItem:nth-child(' + (this.selectedHorizontalItemIndex + 1) + ')').addClass('selected');
    },

    setSelectedHorizontalItem: function () {
        $('.epgItem.selected').removeClass('selected');
        $('.epgRow:nth-child(' + (this.currentItemPositionIndex + 1) + ')' + ' .epgItem:nth-child(' + (this.selectedHorizontalItemIndex + 1) + ')').addClass('selected');
    },

    initializeEpgClock: function () {
        this.updateClock();
        this.clockIntervalId = setInterval(this.updateClock.bind(this), this.clockRefreshInterval);
    },

    updateClock: function () {
        var dt = PluginInterface.Setup.fGetTimeDate().split(":");
        if (dt == '') {
            return false;
        }
        var datetime = new Date(dt[0], dt[1] - 1, dt[2], dt[3], dt[4], dt[5]); //dt[1]-1 because january is representet as 0, february as 1...
        var dateTimeValues = UIMiniFramework.Tools.getDateTimeValuesFromDateObject(datetime, true);
        $('.clockWrapper .day').text(dateTimeValues.DAY_STRING);
        $('.clockWrapper .date').text(dateTimeValues.DAY + '.' + dateTimeValues.MONTH + '.' + dateTimeValues.YEAR + '.');
        $('.clockWrapper .time').text(dateTimeValues.HOUR + ':' + dateTimeValues.MINUTE);
    },

    initializeTimeRangeComponent: function () {
        this.timeLineComponent.nowMilliseconds = UIMiniFramework.Tools.getCurrentDateTimeObject().getTime();
        // round timestamp to nearest half hour; 30 min = 30 * 60 * 1000 ms = 1800000 ms
        var roundedStart = this.timeLineComponent.nowMilliseconds - (this.timeLineComponent.nowMilliseconds % 1800000);
        this.timeLineComponent.startLimitSeconds = roundedStart / 1000;
        this.drawTimeRange(roundedStart);
        this.setTimeOverlayWidth();
        this.initializeTimeOverlay();
    },

    drawTimeRange: function (roundedStart) {
        var start = new Date(roundedStart);
        this.timeLineComponent.startSeconds = start.getTime() / 1000;
        this.timeLineComponent.startString = UIMiniFramework.Tools.getDateTimeStringFromDateObject(start);
        var end = new Date((start.getTime() + this.timeLineComponent.timeLineRange * 60 * 1000));
        var endSeconds = end.getTime() / 1000;
        this.timeLineComponent.endString = UIMiniFramework.Tools.getDateTimeStringFromDateObject(end);
        this.timeLineComponent.endLimitSeconds = endSeconds;
        //console.log(this.timeLineComponent.startString + "..." + this.timeLineComponent.endString);

        var startLimitDate = new Date(this.timeLineComponent.startLimitSeconds * 1000);
        var endLimitDate = new Date(this.timeLineComponent.endLimitSeconds * 1000);
        this.timeLineComponent.daysDiff = UIMiniFramework.Tools.getDateDiffInDays(startLimitDate, endLimitDate);

        var timeLineDate = this.timeLineComponent.startString.substr(8, 2) + '.' + this.timeLineComponent.startString.substr(5, 2) + '.' + this.timeLineComponent.startString.substr(0, 4);
        $('.timeLineDate').text('');

        var i = 0;
        while (i < this.timeLineComponent.timeRangeMarksCount) {
            var time = new Date(roundedStart);
            var hours = (time.getHours() < 10) ? '0' + time.getHours() : time.getHours();
            var minutes = (time.getMinutes() < 10) ? '0' + time.getMinutes() : time.getMinutes();
            $('.timeItem' + i).text(hours + ':' + minutes);
            roundedStart += 1800000;
            if (i > 0 && hours == '00' && minutes == '00') {
                $('.timeLineDate').addClass('leftPosition' + i).text(timeLineDate);
            }
            i++;
        }
    },

    setTimeOverlayWidth: function () {
        this.timeLineComponent.timeOverlayWidth = this.timeLineComponent.oneMinutePixels * ((this.timeLineComponent.nowMilliseconds - this.timeLineComponent.startSeconds * 1000) / 1000 / 60);
        $('.timeOverlay').width(this.timeLineComponent.timeOverlayWidth);
    },

    initializeTimeOverlay: function () {
        clearInterval(this.timeLineComponent.timeOverlayIntervalId);
        this.timeLineComponent.timeOverlayIntervalId = setInterval(this.timeOverlayClock.bind(this), this.timeLineComponent.timeOverlayInterval);
    },

    timeOverlayClock: function () {
        if (this.timeLineComponent.nowMilliseconds >= this.timeLineComponent.endLimitSeconds * 1000) {
            this.shiftHours(1, this.timeLineComponent.hours2ShiftOnePage);
            return;
        }
        this.timeLineComponent.nowMilliseconds += this.timeLineComponent.timeOverlayInterval;
        this.timeLineComponent.timeOverlayWidth = this.timeLineComponent.timeOverlayWidth + (this.timeLineComponent.oneMinutePixels * (this.timeLineComponent.timeOverlayInterval / 1000 / 60));
        $('.timeOverlay').width(this.timeLineComponent.timeOverlayWidth);
    },

    writeEpgItems: function () {
        var i = this.items.length;
        while (i-- > 0) {
            this.writeEpgRow(i);
        }
    },

    writeEpgRow: function (i) {
        var serviceIndex = (this.items[i].index);
        var epgs = PluginInterface.Epg.fGetEpgWindow(serviceIndex, this.timeLineComponent.startString, this.timeLineComponent.endString).split('|');
        var min = parseInt(epgs[0], 10);
        var max = parseInt(epgs[1], 10);
        this.horizontalStartItemIndexes[i] = min;
        this.horizontalItemsCounts[i] = max;
        var epgItemWrapper = $(UIMiniFramework.Templates.tvGuide.epgItemWrapper);
        if (max > 0) {
            for (var j = 0; j < max; j++) {
                var event = PluginInterface.Epg.fGetEpgScheduledData(serviceIndex, min++),
                    epgItem = $(UIMiniFramework.Templates.tvGuide.epgItem),
                    duration = 0,
                    width = 0,
                    eventStartPosition = 0;
                epgItem.text(event.name);
                switch (j) {
                    // first epg item
                    case 0:
                        if (event.startt < this.timeLineComponent.startSeconds) {
                            epgItem.addClass('startsInThePast').append(UIMiniFramework.Templates.tvGuide.leftArrow);
                            duration = (event.endt - this.timeLineComponent.startSeconds) / 60;
                            eventStartPosition = 0;
                        }
                        else {
                            duration = (event.endt - event.startt) / 60;
                            eventStartPosition = ((event.startt - this.timeLineComponent.startSeconds) / 60) * this.timeLineComponent.oneMinutePixels;
                        }
                        // minus 3 pixels for gap between two events but not for the last event in the row
                        width = duration * this.timeLineComponent.oneMinutePixels - (this.timeLineComponent.horizontalEpgEventsGap / 2);
                        break;
                    // last epg item
                    case (epgs[1] - 1):
                        eventStartPosition = ((event.startt - this.timeLineComponent.startSeconds) / 60) * this.timeLineComponent.oneMinutePixels + (this.timeLineComponent.horizontalEpgEventsGap / 2);
                        duration = (event.endt - event.startt) / 60;
                        width = duration * this.timeLineComponent.oneMinutePixels - (this.timeLineComponent.horizontalEpgEventsGap / 2);
                        break;
                    default:
                        eventStartPosition = ((event.startt - this.timeLineComponent.startSeconds) / 60) * this.timeLineComponent.oneMinutePixels + (this.timeLineComponent.horizontalEpgEventsGap / 2);
                        duration = (event.endt - event.startt) / 60;
                        width = duration * this.timeLineComponent.oneMinutePixels - (this.timeLineComponent.horizontalEpgEventsGap);
                        break;
                }

                epgItem.toggleClass('current', this.isEventPresent(event));
                epgItem.toggleClass('scheduled', parseInt(event.isRecordSet, 10) === 1);
                epgItem.toggleClass('remind', parseInt(event.isReminderSet, 10) === 1);
                epgItem.css({
                    'width': width + 'px',
                    'left': eventStartPosition + 'px'
                });
                epgItemWrapper.append(epgItem);
            }
        }
        else {
            epgItemWrapper.addClass('empty');
        }
        $('.epgRow:nth-child(' + (i + 1) + ')').empty().append(epgItemWrapper);
    },

    writeItems: function () {
        var i = this.items.length;
        var isActiveService = false;
        $('.activeChannelIcon').addClass('invisible');
        while (i-- > 0) {
            $('.item' + i + ' .index').text(parseInt(this.items[i].pslIndex, 10));
            $('.item' + i + ' .title').text(this.items[i].name);
            this.setHd(i, this.items[i].type);
            this.setUhd(i, this.items[i].type);
            if (!isActiveService && parseInt(this.items[i].isServiceActive, 10) === 1 && (UIMiniFramework.Globals.service.actuallServiceList == null || UIMiniFramework.Globals.service.actuallServiceList == this.listType)) {
                $('.item' + i + ' .activeChannelIcon').removeClass('invisible');
                isActiveService = true;
            }
        }
    },

    getItem: function () {
        var ret = PluginInterface.ServiceZapper.fGetServiceData(this.listType, this.selectedItemIndex);
        ret.realIndex = this.selectedItemIndex;
        return ret;
    },

    isEventPresent: function (event) {
        return (event.startt <= (this.timeLineComponent.nowMilliseconds / 1000) && event.endt >= (this.timeLineComponent.nowMilliseconds / 1000));
    },

    shiftItems: function (direction) {
        if (direction == 1) {
            this.items.shift();
            this.items.push(this.getItem());
        }
        else {
            this.items.pop();
            this.items.unshift(this.getItem());
        }
        this.createEpgWindow();
    },

    createEpgWindow: function () {
        if (this.items.length == 0) {
            return;
        }
        var chIndexes = "";
        var i = 0;
        for (; i < this.items.length - 1; i++) {
            chIndexes += (this.items[i].index + ":");
        }
        chIndexes += this.items[i].index;
        PluginInterface.Epg.fCreateEPGWindow(this.items.length + ":" + chIndexes, "" + this.timeLineComponent.startString, this.durationWindow);
    },

    shiftHorizontalEpgItem: function (direction, hoursToShift) {
        var isBeginning = false;
        var roundedStart = 0;
        if (direction > 0) {
            // shift by hoursToShift hours in the future (hoursToShift * 3600000 = hoursToShift hours in milliseconds)
            roundedStart = this.timeLineComponent.startSeconds * 1000 + ((hoursToShift * 3600000));
            // if start > 7 days in millisec -> start = max time (start + 7 days) - one page time width
            if (hoursToShift > this.timeLineComponent.hours2ShiftOnePage && roundedStart > this.timeLineComponent.startLimitSeconds * 1000 + (this.maxDaysInFuture * 24 * 3600000)) {
                roundedStart = this.timeLineComponent.startLimitSeconds * 1000 + (this.maxDaysInFuture * 24 * 3600000) - (this.timeLineComponent.hours2ShiftOnePage * 3600000);
            }
        }
        else {
            // shift by hoursToShift hours in the past
            roundedStart = this.timeLineComponent.startSeconds * 1000 - (hoursToShift * 3600000);
            isBeginning = roundedStart <= this.timeLineComponent.startLimitSeconds * 1000 ? true : false;
            if (hoursToShift > this.timeLineComponent.hours2ShiftOnePage && roundedStart < this.timeLineComponent.startLimitSeconds * 1000) {
                roundedStart = this.timeLineComponent.startLimitSeconds * 1000;
            }
        }
        this.drawTimeRange(roundedStart);
        this.writeEpgItems();
        this.calculateHorizontalSelectedItemIndex(direction, isBeginning);
        this.setSelectedItem();
        this.createEpgWindow();
    },

    calculateHorizontalSelectedItemIndex: function (direction, isBeginning) {
        if (direction == 1) {
            // always set horizontal selected index to 0 when moving right in the epg events list
            this.selectedHorizontalItemIndex = 0;
        }
        else {
            if (isBeginning) {
                this.selectedHorizontalItemIndex = 0;
            }
            else {
                // when moving left, set index to max epg event index (epg events count - 1) or 0 if no epg events for selected channel
                this.selectedHorizontalItemIndex = (this.horizontalItemsCounts[this.currentItemPositionIndex] == 0) ? 0 : this.horizontalItemsCounts[this.currentItemPositionIndex] - 1;
            }
        }
    },

    shiftVerticalEpgItem: function (direction) {
        var epgRow = $(UIMiniFramework.Templates.tvGuide.epgRow);
        if (direction == 1) {
            $('.epgRow:nth-child(1)').remove();
            $('.epgWrapper').append(epgRow);

            this.horizontalStartItemIndexes.shift();
            this.horizontalItemsCounts.shift();
            this.horizontalStartItemIndexes.splice(this.horizontalStartItemIndexes.length, 0, 0);
            this.horizontalItemsCounts.splice(this.horizontalItemsCounts.length, 0, 0);

            this.writeEpgRow(this.totalVisibleItems - 1);
        }
        else {
            $('.epgRow:nth-child(' + this.totalVisibleItems + ')').remove();
            $('.epgWrapper').prepend(epgRow);

            this.horizontalStartItemIndexes.pop();
            this.horizontalItemsCounts.pop();
            this.horizontalStartItemIndexes.splice(0, 0, 0);
            this.horizontalItemsCounts.splice(0, 0, 0);

            this.writeEpgRow(0);
        }
        this.setSelectedItem();
    },

    shiftPage: function (direction) {
        if (this.totalItems <= this.totalVisibleItems) {
            return;
        }
        var max = this.totalItems > this.totalVisibleItems ? this.totalVisibleItems : this.totalItems;
        var startIndex = (this.selectedItemIndex - this.currentItemPositionIndex + this.totalVisibleItems * direction + this.totalItems) % this.totalItems;
        for (var i = 0; i < max; i++) {
            this.items[i] = PluginInterface.ServiceZapper.fGetServiceData(this.listType, (startIndex + i) % this.totalItems);
            this.items[i].realIndex = (startIndex + i) % this.totalItems;
        }
        this.selectedItemIndex = (this.selectedItemIndex + this.totalVisibleItems * direction + this.totalItems) % this.totalItems;
        this.writeItems();
        this.writeEpgItems();
        this.selectedHorizontalItemIndex = 0;
        this.setSelectedHorizontalItem();
        this.createEpgWindow();
        setTimeout(this.updateDetails.bind(UIMiniFramework.Modules.TvGuide), 400);
    },

    updateChannelList: function () {
        var channelList = UIMiniFramework.Tools.getArrayObjectByParamValue(this.channelLists, this.listType, 'index');
        channelList = typeof channelList == 'undefined' ? '' : channelList;
        this.el.find('.header .title').text(channelList.label);
    },

    /**
     * update details
     */
    updateDetails: function () {
        if (this.horizontalStartItemIndexes[this.currentItemPositionIndex] >= 0) {
            var event = PluginInterface.Epg.fGetEpgScheduledData(this.items[this.currentItemPositionIndex].index, this.horizontalStartItemIndexes[this.currentItemPositionIndex] + this.selectedHorizontalItemIndex);
            var duration = '(' + event.startdt.substr(11, 5) + ' - ' + event.enddt.substr(11, 5) + ')';
            var status = this.getEventStartDateStatus(event);
            $('.detailsWrapper .title').html('<span class="eventName">' + event.name + '</span>' + '<span class="duration">' + duration + '</span>' + '   ' + '<span class="status">' + status + '</span>');
            $('.detailsWrapper .typeWrapper .value').text(event.genre);
            $('.detailsWrapper .ageWrapper .value').text(event.parental == 0 ? '-' : event.parental);
            $('.detailsWrapper .typeWrapper .label').removeClass('invisible');
            $('.detailsWrapper .ageWrapper .label').removeClass('invisible');
            $('.detailsWrapper .description').text(event.description);
        }
        else {
            this.resetDetails();
        }
        this.toggleInfo();
    },

    resetDetails: function () {
        $('.detailsWrapper .title').text("");
        $('.detailsWrapper .duration').text("");
        $('.detailsWrapper .status').text("");
        $('.detailsWrapper .typeWrapper .value').text("");
        $('.detailsWrapper .ageWrapper .value').text("");
        $('.detailsWrapper .typeWrapper .label').addClass('invisible');
        $('.detailsWrapper .ageWrapper .label').addClass('invisible');
        $('.detailsWrapper .description').text("");
    },

    resetData: function () {
        this.selectedItemIndex = 0;
        this.currentItemPositionIndex = 0;
        this.selectedHorizontalItemIndex = 0;
        this.horizontalItemsCounts.length = 0;
        this.horizontalStartItemIndexes.length = 0;
        this.items = [];

        $('.epgRow .epgItemWrapper').empty().addClass('empty');
        $('.tvGuideItem .index').text("");
        $('.tvGuideItem .title').text("");

        this.resetDetails();
    },

    getEventStartDateStatus: function (event) {
        var startDate = new Date(event.startt * 1000);
        var nowDate = new Date(this.timeLineComponent.nowMilliseconds);
        this.daysDiff = UIMiniFramework.Tools.getDateDiffInDays(nowDate, startDate);
        if (this.daysDiff == 0) {
            return '';
        }
        else if (this.daysDiff == 1) {
            return UIMiniFramework.Translations.tomorrow[UIMiniFramework.Globals.language];
        }
        return event.startdt.substr(8, 2) + '.' + event.startdt.substr(5, 2) + '.' + event.startdt.substr(0, 4);
    },

    setHd: function (itemIndex, value) {
        var isEnabled = false;
        switch (parseInt(value, 10)) {
            case PluginInterface.ServiceZapper.serviceType.TYPE_ADV_CODEC_HD_DIG_TV:
            case PluginInterface.ServiceZapper.serviceType.TYPE_ADV_CODEC_HD_NVOD_TIME_SHIFT:
            case PluginInterface.ServiceZapper.serviceType.TYPE_ADV_CODEC_HD_NVOD_REF:
                isEnabled = true;
                break;
        }
        $('.tvGuideItem.item' + itemIndex).find('.hd').toggleClass('invisible', !isEnabled);
    },

    setUhd: function (itemIndex, value) {
        var isEnabled = false;
        switch (parseInt(value, 10)) {
            case PluginInterface.ServiceZapper.serviceType.TYPE_UHD_4K_DIG_TV:
                isEnabled = true;
                break;
        }
        $('.tvGuideItem.item' + itemIndex).children('.uhd').toggleClass('invisible', !isEnabled);
    },

    showInfo: function () {
        var event = PluginInterface.Epg.fGetEpgScheduledData(this.items[this.currentItemPositionIndex].index, this.horizontalStartItemIndexes[this.currentItemPositionIndex] + this.selectedHorizontalItemIndex);
        if (parseInt(event.id, 10) !== 0) {
            UIMiniFramework.Modules.Info.create(this, event);
            UIMiniFramework.Modules.Info.show();
        }
    },

    shiftEpgDay: function (direction) {
        this.direction = direction;
        this.resetDetails();
        this.shiftHorizontalEpgItem(direction, this.timeLineComponent.hours2Shift24);
        this.setTimeOverlayWidth();
    },

    shiftHours: function (direction, hours) {
        this.direction = direction;
        this.resetDetails();
        this.shiftHorizontalEpgItem(direction, hours);
        this.setTimeOverlayWidth();
    },

    isSkipped: function () {
        if (this.items[this.currentItemPositionIndex].isSkipped) {
            return true;
        }
        return false;
    },

    updateActiveServiceIcon: function (channelIndex) {
        $('.activeChannelIcon').addClass('invisible');
        // if not on hidden service
        if ((UIMiniFramework.Globals.service.actuallServiceList == null || UIMiniFramework.Globals.service.actuallServiceList == this.listType)) {
            for (var i in this.items) {
                this.items[i].isServiceActive = 0;
                if (this.items[i].realIndex == channelIndex) {
                    this.items[i].isServiceActive = 1;
                    $('.item' + i + ' .activeChannelIcon').removeClass('invisible');
                    break;
                }
            }
        }
    },

    isPresentEvent: function () {
        return (this.selectedHorizontalItemIndex == 0 && this.timeLineComponent.startLimitSeconds == this.timeLineComponent.startSeconds);
    },

    toggleInfo: function () {
        $('.tvGuide .infoWrapper').toggleClass('hidden', $('.epgRow.selected .epgItemWrapper').hasClass('empty'));
    },

    setActiveServiceFlag: function () {
        for (var i in this.items) {
            this.items[i].isServiceActive = 0;
        }
        this.items[this.currentItemPositionIndex].isServiceActive = 1;
    },

    ok: function () {
        this.setActiveServiceFlag();
        UIMiniFramework.Globals.service.actuallServiceList = null;
        UIMiniFramework.Modules.ZapperWrapper.setActiveService(this.listType, this.selectedItemIndex);
    },

    applyFilters: function (listIndex) {
        this.resetData();
        this.listType = listIndex;
        this.updateChannelList();
        this.updateItems();
        this.writeItems();
        this.writeEpgItems();
        this.updateDetails();
        this.setSelectedItem();
    },

    zapperCallback: function(components, isError){
        if (this.isCreated){
            this.scaleLiveRoot(200);
            this.updateActiveServiceIcon(UIMiniFramework.Globals.service.channelIndex);
        }
    },

    /**
     * Record
     */
    red: function () {
        if (!UIMiniFramework.Globals.service.isZapped) {
            return;
        }
        if (!PluginInterface.Setup.getPvrActivated()) {
            UIMiniFramework.Tools.recordingDisabledInformation();
            return;
        }
        if (!UIMiniFramework.Globals.isSignalAvailable) {
            UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.noSignal[UIMiniFramework.Globals.language]);
            return;
        }
        if (!UIMiniFramework.Modules.UsbDevices.recorder.isMounted) {
            UIMiniFramework.Modules.UsbDevices.usbNotReady();
            return;
        }
        var isRecordingSet = $('.epgItem.selected').hasClass('scheduled');
        if (!isRecordingSet) {
            var eventIndex = this.horizontalStartItemIndexes[this.currentItemPositionIndex] + this.selectedHorizontalItemIndex;
            if (eventIndex < 0) {
                //console.log("bad index");
                return;
            }
            var realChIndex = PluginInterface.ServiceZapper.fGetServiceData(this.listType, this.selectedItemIndex).index;
            var data = PluginInterface.Epg.fGetEpgScheduledData(realChIndex, eventIndex);
            if (data.uhdEventExist != 0) {
                UIMiniFramework.Modules.Confirmations.add(
                    UIMiniFramework.Translations.popups.recordUhdAvailable[UIMiniFramework.Globals.language],
                    null,
                    function () {
                        UIMiniFramework.Modules.TvGuide.setRecording(true);
                    }, //OK
                    function () {
                        UIMiniFramework.Modules.TvGuide.setRecording(false);
                    }, //cancel,
                    false,
                    false,
                    false,
                    null,
                    null,
                    false
                );
            }
            else {
                this.setRecording(false);
            }
        }
        else {
            UIMiniFramework.Modules.Confirmations.add(
                UIMiniFramework.Translations.popups.doYouWantToDiscardTimer[UIMiniFramework.Globals.language],
                null,
                UIMiniFramework.Modules.TvGuide.setRecording.bind(UIMiniFramework.Modules.TvGuide),
                null,
                false,
                false,
                false,
                null,
                null,
                false
            );
        }
    },

    setRecording: function (setUhdEvent) {
        var eventIndex = this.horizontalStartItemIndexes[this.currentItemPositionIndex] + this.selectedHorizontalItemIndex;
        if (eventIndex < 0) {
            //console.log("bad index");
            return;
        }
        var realChIndex = PluginInterface.ServiceZapper.fGetServiceData(this.listType, this.selectedItemIndex).index;
        var formattedStringArgument = PluginInterface.Epg.fGetEpgScheduledDataString(realChIndex, eventIndex);   //TODO: use fGetEpgScheduledData function instead of this!
        // disable add padding for present event
        if (!this.isPresentEvent()) {
            formattedStringArgument = UIMiniFramework.Modules.Pvr.addPvrPaddings(formattedStringArgument);
        }

        // process recording lack of storage calculation
        var durationString = formattedStringArgument.split('|')[7];
        var durationSeconds = UIMiniFramework.Tools.getSecondsFromTimeString(durationString);
        var serviceType = UIMiniFramework.ServiceListCache.serviceList[this.listType][this.selectedItemIndex].type;
        // only check lack of storage when creating scheduled records, not when delete
        if (!$('.epgItem.selected').hasClass('scheduled')) {
            if (UIMiniFramework.Modules.Pvr.isRecordingLackOfStorage(durationSeconds, serviceType)) {
                return;
            }
        }

        // update schedule title
        var serviceName = this.items[this.currentItemPositionIndex].name;
        var dataArray = formattedStringArgument.split('|');
        var title = dataArray[0];
        dataArray[0] = serviceName + ' ' + title; // title updated
        formattedStringArgument = dataArray.join('|');

        var result;
        if (setUhdEvent) {
            result = PluginInterface.Pvr.fCreateTimerRecord(formattedStringArgument, parseInt(dataArray[14]), PluginInterface.ServiceZapper.serviceList.MASTER_LIST);
        }
        else {
            result = PluginInterface.Pvr.fCreateTimerRecord(formattedStringArgument, this.selectedItemIndex, this.listType);
        }
        ui.App.Callbacks.createTimerRecordResponse(result);
        switch (parseInt(result, 10)) {
            case PluginInterface.Pvr.eScheduleRecordingStatus.SUCCESS:
                $('.epgItem.selected').addClass('scheduled');
                break;
            case PluginInterface.Pvr.eScheduleRecordingStatus.ERROR:
                break;
            case PluginInterface.Pvr.eScheduleRecordingStatus.REMOVED:
                $('.epgItem.selected').removeClass('scheduled');
                break;
        }
    },

    /**
     * Remind
     */
    green: function () {
        if (!UIMiniFramework.Globals.service.isZapped) {
            return;
        }
        if (!UIMiniFramework.Globals.isSignalAvailable) {
            UIMiniFramework.Modules.Notifications.add(UIMiniFramework.Translations.noSignal[UIMiniFramework.Globals.language]);
            return;
        }
        var isReminderSet = $('.epgItem.selected').hasClass('remind');
        if (!isReminderSet) {
            this.setReminder();
        }
        else {
            UIMiniFramework.Modules.Confirmations.add(
                UIMiniFramework.Translations.popups.doYouWantToDiscardTimer[UIMiniFramework.Globals.language],
                null,
                UIMiniFramework.Modules.TvGuide.setReminder.bind(UIMiniFramework.Modules.TvGuide),
                null,
                false,
                false,
                false,
                null,
                null,
                false
            );
        }
    },

    setReminder: function () {
        var eventIndex = this.horizontalStartItemIndexes[this.currentItemPositionIndex] + this.selectedHorizontalItemIndex;
        if (eventIndex < 0) {
            //console.log("bad index");
            return;
        }
        var realChIndex;
        switch (this.listType) {
            case PluginInterface.ServiceZapper.serviceList.MASTER_LIST:
            //realChIndex = this.selectedItemIndex;
            //break;
            case PluginInterface.ServiceZapper.serviceList.FAVORITE_LIST1:
            case PluginInterface.ServiceZapper.serviceList.FAVORITE_LIST2:
            case PluginInterface.ServiceZapper.serviceList.FAVORITE_LIST3:
            case PluginInterface.ServiceZapper.serviceList.FAVORITE_LIST4:
            case PluginInterface.ServiceZapper.serviceList.FAVORITE_LIST5:
            case PluginInterface.ServiceZapper.serviceList.TV_LIST:
            case PluginInterface.ServiceZapper.serviceList.RADIO_LIST:
                realChIndex = PluginInterface.ServiceZapper.fGetServiceData(this.listType, this.selectedItemIndex).index;
                break;
        }
        var formattedStringArgument = PluginInterface.Epg.fGetEpgScheduledDataString(realChIndex, eventIndex); //TODO: use fGetEpgScheduledData function instead of this!
        // update schedule title
        var serviceName = this.items[this.currentItemPositionIndex].name;
        var dataArray = formattedStringArgument.split('|');
        var title = dataArray[0];
        dataArray[0] = serviceName + ' ' + title; // title updated
        formattedStringArgument = dataArray.join('|');

        var result = PluginInterface.Reminder.fSetReminderData(formattedStringArgument, this.selectedItemIndex, this.listType);
        ui.App.Callbacks.setReminderDataResponse(result);
        switch (parseInt(result, 10)) {
            case PluginInterface.Reminder.eScheduleReminderStatus.SUCCESS:
                $('.epgItem.selected').addClass('remind');
                break;
            case PluginInterface.Reminder.eScheduleReminderStatus.REMOVED:
                $('.epgItem.selected').removeClass('remind');
                break;
        }
    },

    yellow: function () {
        UIMiniFramework.Modules.TvGuideFilters.create();
        UIMiniFramework.Modules.TvGuideFilters.show();
    },

    handleKeyPress: function (keyCode) {
        switch (keyCode) {
            case UIMiniFramework.Keys.OK:
                this.ok();
                return true;
            case UIMiniFramework.Keys.Left:
                this.shiftHorizontal(-1);
                return true;
            case UIMiniFramework.Keys.Right:
                this.shiftHorizontal(1);
                return true;
            case UIMiniFramework.Keys.Up:
                this.shiftVertical(-1);
                return true;
            case UIMiniFramework.Keys.Down:
                this.shiftVertical(1);
                return true;
            case UIMiniFramework.Keys.Exit:
                this.destroy(true);
                return true;
            case UIMiniFramework.Keys.Back:
                this.destroy();
                return true;
            case UIMiniFramework.Keys.Info:
                this.showInfo();
                return true;
            case UIMiniFramework.Keys.Green:
                this.green();
                return true;
            case UIMiniFramework.Keys.Red:
                this.red();
                return true;
            case UIMiniFramework.Keys.Blue:
                break;
            case UIMiniFramework.Keys.Yellow:
                this.yellow();
                return true;
            case UIMiniFramework.Keys.ChannelUp:
                UIMiniFramework.Modules.ZapperWrapper.setNextService();
                return true;
            case UIMiniFramework.Keys.ChannelDown:
                UIMiniFramework.Modules.ZapperWrapper.setPreviousService();
                return true;
            case UIMiniFramework.Keys.PgUp:
                this.shiftPage(-1);
                return true;
            case UIMiniFramework.Keys.PgDown:
                this.shiftPage(1);
                return true;
            case UIMiniFramework.Keys.FastRewind:
                this.shiftEpgDay(-1);
                return true;
            case UIMiniFramework.Keys.FastForward:
                this.shiftEpgDay(1);
                return true;
            default:
                return false;
        }
    }
};