'use strict';

UIMiniFramework.Modules.TvGuideFilters = {
    GUID: UIMiniFramework.Tools.getGUID(),
    wrapperSelector: UIMiniFramework.Config.uiAppWrapperSelector,
    el: null,
    isCreated: false,
    isShown: false,
    components: [],

    create: function () {
        this.init();
        this.render();
        this.hide();
        this.addToDOM();
        this.updateLabels();
        this.focus();
        this.isCreated = true;
        this.isShown = false;
    },

    destroy: function (exit) {
        this.blur();
        this.removeFromDOM();
        this.isCreated = false;
        this.isShown = false;
        this.components = [];
        if (exit) {
            UIMiniFramework.Modules.TvGuide.destroy();
        }
    },

    init: function () {
        this.components = new ui.Mappings.EpgChangeFilters(UIMiniFramework.Modules.TvGuideFilters);
    },

    render: function () {
        this.el = $(UIMiniFramework.Templates.tvGuide.changeFilters);
        if (this.components && this.components.renderComponents) {
            this.components.renderComponents();
        }
    },

    updateLabels: function () {
        if (this.components && this.components.updateComponentLabels) {
            this.components.updateComponentLabels();
        }
    },

    addToDOM: function () {
        $(this.wrapperSelector).append(this.el);
    },

    show: function () {
        this.el.removeClass('invisible');
        this.isShown = true;
    },

    hide: function () {
        this.el.addClass('invisible');
        this.isShown = false;
    },

    focus: function () {
        UIMiniFramework.Modules.Input.currentFocus = this;
    },

    blur: function () {
        UIMiniFramework.Modules.Input.currentFocus = UIMiniFramework.Modules.TvGuide;
    },

    removeFromDOM: function () {
        this.el.remove();
        this.el = null;
    },

    toggle: function (isVisible) {
        if (this.isCreated) {
            if (isVisible) {
                this.show()
            }
            else {
                this.hide();
            }
        }
    },

    ok: function(){
        UIMiniFramework.Modules.TvGuide.applyFilters(this.components.items[0].keys[this.components.items[0].selectedItemIndex]);
        this.destroy();
    },

    exit: function(keyCode){
        this.destroy();
        UIMiniFramework.Modules.TvGuide.handleKeyPress(keyCode);
    },

    handleKeyPress: function (keyCode) {
        switch (keyCode) {
            case UIMiniFramework.Keys.OK:
                this.ok();
                return true;
            case UIMiniFramework.Keys.Left:
                this.components.shiftHorizontal(-1);
                return true;
            case UIMiniFramework.Keys.Right:
                this.components.shiftHorizontal(1);
                return true;
            case UIMiniFramework.Keys.Up:
                this.components.shiftVertical(-1);
                return true;
            case UIMiniFramework.Keys.Down:
                this.components.shiftVertical(1);
                return true;
            case UIMiniFramework.Keys.Exit:
                this.exit();
                return true;
            case UIMiniFramework.Keys.Back:
                this.destroy();
                return true;
            default:
                return false;
        }
    }
};